"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.CmdletClass = exports.NewAddInfoAttribute = exports.addInfoAttribute = exports.addDefaultInfo = exports.addParameterPreviewMessage = exports.addParameterBreakingChange = exports.addPSArgumentCompleterAttribute = exports.isEnumImplementation = exports.NewAddCompleterInfo = exports.addCompleterInfo = exports.createStepper = void 0;
const ejs = require('ejs');
const codemodel_1 = require("@autorest/codemodel");
const codemodel_v3_1 = require("@azure-tools/codemodel-v3");
const command_operation_1 = require("../utils/command-operation");
const schema_1 = require("../utils/schema");
const codegen_1 = require("@azure-tools/codegen");
const linq_1 = require("@azure-tools/linq");
const codegen_csharp_1 = require("@azure-tools/codegen-csharp");
const exports_1 = require("../llcsharp/exports");
const powershell_declarations_1 = require("../internal/powershell-declarations");
const extension_base_1 = require("@autorest/extension-base");
const codegen_csharp_2 = require("@azure-tools/codegen-csharp");
const model_class_1 = require("../llcsharp/model/model-class");
const resourceName_1 = require("../utils/resourceName");
const PropertiesRequiringNew = new Set(['Host', 'Events']);
const Verbs = {
    Common: 'global::System.Management.Automation.VerbsCommon',
    Data: 'global::System.Management.Automation.VerbsData',
    Lifecycle: 'global::System.Management.Automation.VerbsLifecycle',
    Diagnostic: 'global::System.Management.Automation.VerbsDiagnostic',
    Communications: 'global::System.Management.Automation.VerbsCommunications',
    Security: 'global::System.Management.Automation.VerbsSecurity',
    Other: 'global::System.Management.Automation.VerbsOther'
};
const category = {
    'Add': Verbs.Common,
    'Clear': Verbs.Common,
    'Close': Verbs.Common,
    'Copy': Verbs.Common,
    'Enter': Verbs.Common,
    'Exit': Verbs.Common,
    'Find': Verbs.Common,
    'Format': Verbs.Common,
    'Get': Verbs.Common,
    'Hide': Verbs.Common,
    'Join': Verbs.Common,
    'Lock': Verbs.Common,
    'Move': Verbs.Common,
    'New': Verbs.Common,
    'Open': Verbs.Common,
    'Optimize': Verbs.Common,
    'Pop': Verbs.Common,
    'Push': Verbs.Common,
    'Redo': Verbs.Common,
    'Remove': Verbs.Common,
    'Rename': Verbs.Common,
    'Reset': Verbs.Common,
    'Resize': Verbs.Common,
    'Search': Verbs.Common,
    'Select': Verbs.Common,
    'Set': Verbs.Common,
    'Show': Verbs.Common,
    'Skip': Verbs.Common,
    'Split': Verbs.Common,
    'Step': Verbs.Common,
    'Switch': Verbs.Common,
    'Undo': Verbs.Common,
    'Unlock': Verbs.Common,
    'Watch': Verbs.Common,
    'Backup': Verbs.Data,
    'Checkpoint': Verbs.Data,
    'Compare': Verbs.Data,
    'Compress': Verbs.Data,
    'Convert': Verbs.Data,
    'ConvertFrom': Verbs.Data,
    'ConvertTo': Verbs.Data,
    'Dismount': Verbs.Data,
    'Edit': Verbs.Data,
    'Expand': Verbs.Data,
    'Export': Verbs.Data,
    'Group': Verbs.Data,
    'Import': Verbs.Data,
    'Initialize': Verbs.Data,
    'Limit': Verbs.Data,
    'Merge': Verbs.Data,
    'Mount': Verbs.Data,
    'Out': Verbs.Data,
    'Publish': Verbs.Data,
    'Restore': Verbs.Data,
    'Save': Verbs.Data,
    'Sync': Verbs.Data,
    'Unpublish': Verbs.Data,
    'Update': Verbs.Data,
    'Approve': Verbs.Lifecycle,
    'Assert': Verbs.Lifecycle,
    'Complete': Verbs.Lifecycle,
    'Confirm': Verbs.Lifecycle,
    'Deny': Verbs.Lifecycle,
    'Disable': Verbs.Lifecycle,
    'Enable': Verbs.Lifecycle,
    'Install': Verbs.Lifecycle,
    'Invoke': Verbs.Lifecycle,
    'Register': Verbs.Lifecycle,
    'Request': Verbs.Lifecycle,
    'Restart': Verbs.Lifecycle,
    'Resume': Verbs.Lifecycle,
    'Start': Verbs.Lifecycle,
    'Stop': Verbs.Lifecycle,
    'Submit': Verbs.Lifecycle,
    'Suspend': Verbs.Lifecycle,
    'Uninstall': Verbs.Lifecycle,
    'Unregister': Verbs.Lifecycle,
    'Wait': Verbs.Lifecycle,
    'Debug': Verbs.Diagnostic,
    'Measure': Verbs.Diagnostic,
    'Ping': Verbs.Diagnostic,
    'Repair': Verbs.Diagnostic,
    'Resolve': Verbs.Diagnostic,
    'Test': Verbs.Diagnostic,
    'Trace': Verbs.Diagnostic,
    'Connect': Verbs.Communications,
    'Disconnect': Verbs.Communications,
    'Read': Verbs.Communications,
    'Receive': Verbs.Communications,
    'Send': Verbs.Communications,
    'Write': Verbs.Communications,
    'Block': Verbs.Security,
    'Grant': Verbs.Security,
    'Protect': Verbs.Security,
    'Revoke': Verbs.Security,
    'Unblock': Verbs.Security,
    'Unprotect': Verbs.Security,
    'Use': Verbs.Other,
};
function createStepper(p) {
    return (0, codegen_csharp_1.toExpression)(`${codegen_csharp_1.System.Linq.Enumerable.declaration}.Select<${exports_1.ClientRuntime.SendAsyncStep.declaration},${exports_1.ClientRuntime.SendAsyncStep.declaration}>(${p.value}, step =>
    (r, c, n) => {
      ${powershell_declarations_1.DefaultRunspace.value} = ${powershell_declarations_1.DefaultRunspace.value} ?? ${powershell_declarations_1.RunspaceFactory.declaration}.CreateRunspace();
      return step(r, c, n);
    })`);
}
exports.createStepper = createStepper;
function addCompleterInfo(targetProperty, parameter) {
    if (parameter.completerInfo && parameter.completerInfo.script) {
        targetProperty.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.CompleterInfoAttribute, {
            parameters: [
                new codegen_csharp_1.LiteralExpression(`\nName = ${new codegen_csharp_1.StringExpression(parameter.completerInfo.name || '').value}`),
                new codegen_csharp_1.LiteralExpression(`\nDescription =${new codegen_csharp_1.StringExpression(parameter.completerInfo.description || '').value}`),
                new codegen_csharp_1.LiteralExpression(`\nScript = ${new codegen_csharp_1.StringExpression(parameter.completerInfo.script).value}`)
            ]
        }));
    }
}
exports.addCompleterInfo = addCompleterInfo;
function NewAddCompleterInfo(targetProperty, parameter) {
    if (parameter.completerInfo && parameter.completerInfo.script) {
        targetProperty.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.CompleterInfoAttribute, {
            parameters: [
                new codegen_csharp_1.LiteralExpression(`\nName = ${new codegen_csharp_1.StringExpression(parameter.completerInfo.name || '').value}`),
                new codegen_csharp_1.LiteralExpression(`\nDescription =${new codegen_csharp_1.StringExpression(parameter.completerInfo.description || '').value}`),
                new codegen_csharp_1.LiteralExpression(`\nScript = ${new codegen_csharp_1.StringExpression(parameter.completerInfo.script).value}`)
            ]
        }));
    }
}
exports.NewAddCompleterInfo = NewAddCompleterInfo;
function isEnumImplementation(schema) {
    return (schema?.type === codemodel_1.SchemaType.SealedChoice && !schema.language.default.skip) ||
        (schema?.extensions && schema.extensions['x-ms-enum']);
}
exports.isEnumImplementation = isEnumImplementation;
function addPSArgumentCompleterAttribute(targetProperty, parameterSchema) {
    const enumValues = (0, linq_1.values)(parameterSchema.language.csharp.enum.values).select(v => `"${v.value}"`).toArray().join(', ');
    targetProperty.add(new codegen_csharp_1.Attribute(powershell_declarations_1.PSArgumentCompleterAttribute, { parameters: [`${enumValues}`] }));
}
exports.addPSArgumentCompleterAttribute = addPSArgumentCompleterAttribute;
function addParameterBreakingChange(targetProperty, parameter) {
    if (parameter.breakingChange) {
        const parameters = [];
        parameters.push(`"${parameter.breakingChange.parameterName}"`);
        if (!parameter.breakingChange.deprecateByVersion || !parameter.breakingChange.deprecateByAzVersion) {
            throw new Error(`breakingChange.deprecateByVersion and breakingChange.deprecateByAzVersion must be set for ${parameter.name}`);
        }
        parameters.push(`"${parameter.breakingChange.deprecateByAzVersion}"`);
        parameters.push(`"${parameter.breakingChange.deprecateByVersion}"`);
        if (parameter.breakingChange.changeInEfectByDate)
            parameters.push(`"${parameter.breakingChange.changeInEfectByDate}"`);
        if (parameter.breakingChange.replacement)
            parameters.push(`ReplaceMentCmdletParameterName="${parameter.breakingChange.replacement}"`);
        if (parameter.breakingChange.isBecomingMandatory)
            parameters.push(`IsBecomingMandatory=${parameter.breakingChange.isBecomingMandatory}`);
        if (parameter.breakingChange.changeDescription)
            parameters.push(`ChangeDescription="${parameter.breakingChange.changeDescription}"`);
        if (parameter.breakingChange.newParameterType) {
            // If old type is set in directive, use it, otherwise try to get the type from the schema
            if (parameter.breakingChange.oldParamaterType) {
                parameters.push(`OldParamaterType="${parameter.breakingChange.oldParamaterType}"`);
            }
            else {
                // for primitive types, should use name, otherwise use fullname which contains namespace
                parameters.push(`OldParamaterType="${parameter.schema.language.csharp.fullname.startsWith('<') ? parameter.schema.language.csharp.name : parameter.schema.language.csharp.fullname}"`);
            }
            parameters.push(`NewParameterType="${parameter.breakingChange.newParameterType}"`);
        }
        targetProperty.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.ParameterBreakingChangeAttribute, {
            parameters: parameters
        }));
    }
}
exports.addParameterBreakingChange = addParameterBreakingChange;
function addParameterPreviewMessage(targetProperty, parameter) {
    if (parameter.previewAnnouncement) {
        const parameters = [];
        parameters.push(`"${parameter.previewAnnouncement.previewMessage}"`);
        if (parameter.previewAnnouncement.estimatedGaDate)
            parameters.push(`"${parameter.previewAnnouncement.estimatedGaDate}"`);
        targetProperty.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.PreviewMessageAttribute, { parameters: parameters }));
    }
}
exports.addParameterPreviewMessage = addParameterPreviewMessage;
function addDefaultInfo(targetProperty, parameter) {
    if (parameter.defaultInfo && parameter.defaultInfo.script) {
        targetProperty.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.DefaultInfoAttribute, {
            parameters: [
                new codegen_csharp_1.LiteralExpression(`\nName = ${new codegen_csharp_1.StringExpression(parameter.defaultInfo.name || '').value}`),
                new codegen_csharp_1.LiteralExpression(`\nDescription =${new codegen_csharp_1.StringExpression(parameter.defaultInfo.description || '').value}`),
                new codegen_csharp_1.LiteralExpression(`\nScript = ${new codegen_csharp_1.StringExpression(parameter.defaultInfo.script).value}`),
                new codegen_csharp_1.LiteralExpression(`\nSetCondition = ${new codegen_csharp_1.StringExpression(parameter.defaultInfo['set-condition'] || '').value}`)
            ]
        }));
    }
}
exports.addDefaultInfo = addDefaultInfo;
function addInfoAttribute(targetProperty, pType, isRequired, isReadOnly, description, serializedName) {
    let pt = pType;
    while (pt.elementType) {
        switch (pt.elementType.schema.type) {
            case codemodel_v3_1.JsonType.Object:
                if (pt.elementType.schema.details.csharp.interfaceImplementation) {
                    pt = {
                        declaration: pt.elementType.schema.details.csharp.interfaceImplementation.declaration,
                        schema: pt.elementType.schema,
                    };
                }
                else {
                    // arg! it's not done yet. Hope it's not polymorphic itself.
                    pt = {
                        declaration: `${pt.elementType.schema.details.csharp.namespace}.${pt.elementType.schema.details.csharp.interfaceName}`,
                        schema: pt.elementType.schema,
                    };
                }
                break;
            case codemodel_v3_1.JsonType.Array:
                pt = pt.elementType;
                break;
            default:
                pt = pt.elementType;
                break;
        }
    }
    const ptypes = new Array();
    if (pt.schema && pt.schema && pt.schema.details.csharp.byReference) {
        ptypes.push(`typeof(${pt.schema.details.csharp.namespace}.${pt.schema.details.csharp.interfaceName}_Reference)`);
        // do we need polymorphic types for by-resource ? Don't think so.
    }
    else {
        ptypes.push(`typeof(${pt.declaration})`);
        if (pt.schema && pt.schema.details.csharp.classImplementation && pt.schema.details.csharp.classImplementation.discriminators) {
            ptypes.push(...[...pt.schema.details.csharp.classImplementation.discriminators.values()].map(each => `typeof(${each.modelInterface.fullName})`));
        }
    }
    targetProperty.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.InfoAttribute, {
        parameters: [
            new codegen_csharp_1.LiteralExpression(`\nRequired = ${isRequired}`),
            new codegen_csharp_1.LiteralExpression(`\nReadOnly = ${isReadOnly}`),
            new codegen_csharp_1.LiteralExpression(`\nDescription = ${new codegen_csharp_1.StringExpression(description).value}`),
            new codegen_csharp_1.LiteralExpression(`\nSerializedName = ${new codegen_csharp_1.StringExpression(serializedName).value}`),
            new codegen_csharp_1.LiteralExpression(`\nPossibleTypes = new [] { ${ptypes.join(',').replace(/\?/g, '').replace(/undefined\./g, '')} }`),
        ]
    }));
}
exports.addInfoAttribute = addInfoAttribute;
function NewAddInfoAttribute(targetProperty, pType, isRequired, isReadOnly, description, serializedName) {
    let pt = pType;
    while (pt.elementType) {
        switch (pt.elementType.schema.type) {
            case codemodel_v3_1.JsonType.Object:
                if (pt.elementType.schema.language.csharp.interfaceImplementation) {
                    pt = {
                        declaration: pt.elementType.schema.language.csharp.interfaceImplementation.declaration,
                        schema: pt.elementType.schema,
                    };
                }
                else {
                    // arg! it's not done yet. Hope it's not polymorphic itself.
                    pt = {
                        declaration: `${pt.elementType.schema.language.csharp.namespace}.${pt.elementType.schema.language.csharp.interfaceName}`,
                        schema: pt.elementType.schema,
                    };
                }
                break;
            case codemodel_v3_1.JsonType.Array:
                pt = pt.elementType;
                break;
            default:
                pt = pt.elementType;
                break;
        }
    }
    const ptypes = new Array();
    if (pt.schema && pt.schema && pt.schema.language.csharp.byReference) {
        ptypes.push(`typeof(${pt.schema.language.csharp.namespace}.${pt.schema.language.csharp.interfaceName}_Reference)`);
        // do we need polymorphic types for by-resource ? Don't think so.
    }
    else {
        ptypes.push(`typeof(${pt.declaration})`);
        if (pt.schema && pt.schema.language.csharp.classImplementation && pt.schema.language.csharp.classImplementation.discriminators) {
            ptypes.push(...[...pt.schema.language.csharp.classImplementation.discriminators.values()].map(each => `typeof(${each.modelInterface.fullName})`));
        }
    }
    targetProperty.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.InfoAttribute, {
        parameters: [
            new codegen_csharp_1.LiteralExpression(`\nRequired = ${isRequired}`),
            new codegen_csharp_1.LiteralExpression(`\nReadOnly = ${isReadOnly}`),
            new codegen_csharp_1.LiteralExpression(`\nDescription = ${new codegen_csharp_1.StringExpression(description).value}`),
            new codegen_csharp_1.LiteralExpression(`\nSerializedName = ${new codegen_csharp_1.StringExpression(serializedName).value}`),
            new codegen_csharp_1.LiteralExpression(`\nPossibleTypes = new [] { ${ptypes.join(',').replace(/\?/g, '').replace(/undefined\./g, '')} }`),
        ]
    }));
}
exports.NewAddInfoAttribute = NewAddInfoAttribute;
class CmdletClass extends codegen_csharp_1.Class {
    constructor(namespace, operation, state, objectInitializer) {
        // generate the 'variant'  part of the name
        const noun = `${state.project.prefix}${operation.details.csharp.subjectPrefix}${operation.details.csharp.subject}`;
        const variantName = `${noun}${operation.details.csharp.name ? `_${operation.details.csharp.name}` : ''}`;
        const name = `${operation.details.csharp.verb}${variantName}`;
        super(namespace, name, powershell_declarations_1.PSCmdlet);
        this.dropBodyParameter = operation.details.csharp.dropBodyParameter ? true : false;
        this.apply(objectInitializer);
        this.operation = operation;
        this.apiCall = this.operation.callGraph[this.operation.callGraph.length - 1];
        // create the response handlers
        this.responses = [...(0, linq_1.values)(this.apiCall.responses), ...(0, linq_1.values)(this.apiCall.exceptions)];
        this.callbackMethods = (0, linq_1.values)(this.responses).toArray().map(each => new codegen_csharp_1.LiteralExpression(each.language.csharp?.name || ''));
        this.operationParameters = [];
        this.state = state;
        this.thingsToSerialize = [];
        this.variantName = variantName;
        this.hasStreamOutput = false;
        this.interfaces.push(exports_1.ClientRuntime.IEventListener);
        this.interfaces.push(exports_1.ClientRuntime.IContext);
        this.eventListener = new exports_1.EventListener(new codegen_csharp_1.LiteralExpression(`((${exports_1.ClientRuntime.IEventListener})this)`), true);
        this.isViaIdentity = variantName.indexOf('ViaIdentity') > 0;
        this.clientsidePagination = !!operation.details.csharp.clientsidePagination && !!this.apiCall.language.csharp?.pageable;
        this.inputObjectParameterName = 'InputObject';
    }
    async init() {
        // basic stuff
        this.addCommonStuff();
        this.description = (0, codegen_1.escapeString)(this.operation.details.csharp.description);
        const $this = this;
        this.add(new codegen_csharp_1.Method('BeginProcessing', codegen_csharp_1.dotnet.Void, {
            override: codegen_csharp_1.Modifier.Override,
            access: codegen_csharp_1.Access.Protected,
            description: `(overrides the default BeginProcessing method in ${powershell_declarations_1.PSCmdlet})`,
            *body() {
                if ($this.state.project.azure) {
                    yield `var telemetryId = ${$this.state.project.serviceNamespace.moduleClass.declaration}.Instance.GetTelemetryId.Invoke();`;
                    yield (0, codegen_csharp_1.If)('telemetryId != "" && telemetryId != "internal"', '__correlationId = telemetryId;');
                }
                yield 'Module.Instance.SetProxyConfiguration(Proxy, ProxyCredential, ProxyUseDefaultCredentials);';
                yield (0, codegen_csharp_1.If)($this.$('Break'), `${exports_1.ClientRuntime.AttachDebugger}.Break();`);
                yield $this.eventListener.syncSignal(powershell_declarations_1.Events.CmdletBeginProcessing);
            }
        }));
        // construct the class
        this.NewAddClassAttributes(this.operation, this.variantName);
        if (this.hasStreamOutput) {
            this.outFileParameter = this.add(new codegen_csharp_1.Property('OutFile', codegen_csharp_1.System.String, { attributes: [], description: 'Path to write output file to.' }));
            this.outFileParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = true', 'HelpMessage = "Path to write output file to"'] }));
            this.outFileParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ValidateNotNull));
            this.outFileParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Body`] }));
        }
        this.NewAddPowershellParameters(this.operation);
        // implement IEventListener
        this.implementIEventListener();
        // implement part of the IContext
        this.implementIContext();
        // add constructors
        this.implementConstructors();
        // add callback methods
        this.NewImplementResponseMethod();
        // processRecord
        this.NewImplementProcessRecord();
        // find each parameter to the method, and find out where the value is going to come from.
        this.operationParameters =
            (0, linq_1.values)(this.apiCall.parameters).
                // filter out constants and path parameters when using piping for identity
                where(each => !(each.language.csharp?.constantValue) && each.language.default?.name !== '$host' /* && (!$this.isViaIdentity || each.in !== ParameterLocation.Path) */).
                select(p => {
                return {
                    name: p.language.csharp?.name,
                    param: (0, linq_1.values)(this.properties).
                        where(each => each.metadata.parameterDefinition).
                        first(each => each.metadata.parameterDefinition.language.csharp?.serializedName === p.language.csharp?.serializedName), // xichen: Is it safe enough to use serializedName?
                    parameterLocation: p.protocol?.http?.in
                };
            }).
                select(each => {
                if (each.param) {
                    const httpParam = (each.param.metadata.parameterDefinition);
                    if (httpParam.required) {
                        return {
                            name: each.param,
                            expression: each.param,
                            parameterLocation: each.parameterLocation
                        };
                    }
                    const httpParamTD = this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(httpParam.schema, httpParam.required, this.state);
                    return {
                        name: each.param,
                        expression: (0, codegen_csharp_1.toExpression)(`this.InvocationInformation.BoundParameters.ContainsKey("${each.param.value}") ? ${each.param.value} : ${httpParamTD.defaultOfType}`),
                        parameterLocation: each.parameterLocation
                    };
                }
                return { name: each.name, expression: codegen_csharp_1.dotnet.Null, parameterLocation: each.parameterLocation };
            }).toArray();
        this.NewImplementProcessRecordAsync();
        this.debugMode = await this.state.getValue('debug', false);
        // json serialization
        this.NewImplementSerialization(this.operation);
        for (const prop of this.properties) {
            if (prop.name === 'Host') {
                prop['new'] = codegen_csharp_1.Modifier.New;
            }
        }
        return this;
    }
    get headerComment() {
        const header = super.headerComment;
        let ops = '';
        for (const httpOperation of (0, linq_1.values)(this.operation.callGraph)) {
            const request = httpOperation.requests?.[0];
            if (!request) {
                continue;
            }
            const httpMethod = request.protocol.http?.method ?? '';
            const httpPath = request.protocol.http?.path ?? '';
            ops = `${ops}\n[OpenAPI] ${httpOperation.language.default.name}=>${httpMethod.toUpperCase()}:"${httpPath}"`;
            if (this.debugMode) {
                // x-ms-metadata seems no longer exists
                // const m = (httpOperation.extensions && httpOperation.extensions['x-ms-metadata']) || (httpOperation.pathExtensions ? httpOperation.pathExtensions['x-ms-metadata'] : undefined);
                // if (m) {
                //   ops = `${ops}\n [METADATA]\n${serialize(m)}`;
                // }
                ops = `${ops}\n [DETAILS]`;
                ops = `${ops}\n verb: ${this.operation.details.csharp.verb}`;
                ops = `${ops}\n subjectPrefix: ${this.operation.details.csharp.subjectPrefix}`;
                ops = `${ops}\n subject: ${this.operation.details.csharp.subject}`;
                ops = `${ops}\n variant: ${this.operation.details.csharp.name}`;
            }
        }
        return ops ? `${header}\n${(0, codegen_1.docComment)((0, codegen_csharp_1.xmlize)('remarks', ops))}` : header;
    }
    AddSwitchViewProperty(responseType = codegen_csharp_1.dotnet.Object) {
        const fieldNames = this.fields.map(f => f.name);
        if (!fieldNames.includes('_responseSize')) {
            this.add(new codegen_csharp_1.Field('_responseSize', codegen_csharp_1.dotnet.Int, {
                initialValue: '0', description: 'A flag to tell whether it is the first returned object in a call. Zero means no response yet. One means 1 returned object. Two means multiple returned objects in response.', access: codegen_csharp_1.Access.Private
            }));
        }
        if (!fieldNames.includes('_firstResponse')) {
            this.add(new codegen_csharp_1.Field('_firstResponse', responseType, {
                initialValue: codegen_csharp_1.dotnet.Null, description: 'A buffer to record first returned object in response.', access: codegen_csharp_1.Access.Private
            }));
        }
    }
    FlushResponse(singleFlush = true) {
        const fieldNames = this.fields.map(f => f.name);
        return fieldNames.includes('_responseSize') ?
            (0, codegen_csharp_1.If)('1 ==_responseSize', function* () {
                yield '// Flush buffer';
                if (singleFlush) {
                    yield 'WriteObject(_firstResponse);';
                }
                else {
                    yield 'WriteObject(_firstResponse.AddMultipleTypeNameIntoPSObject());';
                }
            }) : '';
    }
    WriteObjectWithViewControl(valueName, isEnumerable = false) {
        const $this = this;
        if ($this.state.project.autoSwitchView) {
            if (isEnumerable) {
                return function* () {
                    yield (0, codegen_csharp_1.If)(`null != ${valueName}`, function* () {
                        yield (0, codegen_csharp_1.If)(`0 == _responseSize && 1 == ${valueName}.Count`, function* () {
                            yield `_firstResponse = ${valueName}[0];`;
                            yield '_responseSize = 1;';
                        });
                        yield (0, codegen_csharp_1.Else)(function* () {
                            yield $this.FlushResponse(false);
                            yield 'var values = new System.Collections.Generic.List<System.Management.Automation.PSObject>();';
                            yield (0, codegen_csharp_1.ForEach)('value', valueName, function* () {
                                yield 'values.Add(value.AddMultipleTypeNameIntoPSObject());';
                            });
                            yield 'WriteObject(values, true); ';
                            yield '_responseSize = 2;';
                        });
                    });
                };
            }
            else {
                return function* () {
                    yield (0, codegen_csharp_1.If)(`null != ${valueName}`, function* () {
                        yield (0, codegen_csharp_1.If)('0 == _responseSize', function* () {
                            yield `_firstResponse = ${valueName};`;
                            yield '_responseSize = 1;';
                        });
                        yield (0, codegen_csharp_1.Else)(function* () {
                            yield $this.FlushResponse(false);
                            yield `WriteObject(${valueName}.AddMultipleTypeNameIntoPSObject());`;
                            yield '_responseSize = 2;';
                        });
                    });
                };
            }
        }
        else {
            return `WriteObject(${valueName}, ${isEnumerable});`;
        }
    }
    addCommonStuff() {
        // add a private copy of invocation information for our own uses.
        const privateInvocationInfo = this.add(new codegen_csharp_1.Field('__invocationInfo', powershell_declarations_1.InvocationInfo, { description: 'A copy of the Invocation Info (necessary to allow asJob to clone this cmdlet)', access: codegen_csharp_1.Access.Private }));
        this.invocationInfo = new codegen_csharp_1.Property('InvocationInformation', powershell_declarations_1.InvocationInfo, { description: 'Accessor for our copy of the InvocationInfo.' });
        this.invocationInfo.get = (0, codegen_csharp_1.toExpression)(`${privateInvocationInfo.value} = ${privateInvocationInfo.value} ?? this.MyInvocation `);
        this.invocationInfo.set = new codegen_csharp_1.Statements(privateInvocationInfo.assign('value'));
        this.add(this.invocationInfo);
        if (this.state.project.azure) {
            this.correlationId = this.add(new codegen_csharp_1.Field('__correlationId', codegen_csharp_1.dotnet.String, { initialValue: 'System.Guid.NewGuid().ToString()', description: 'A unique id generatd for the this cmdlet when it is instantiated.', access: codegen_csharp_1.Access.Private }));
            this.processRecordId = this.add(new codegen_csharp_1.Field('__processRecordId', codegen_csharp_1.dotnet.String, { description: 'A unique id generatd for the this cmdlet when ProcessRecord() is called.', access: codegen_csharp_1.Access.Private }));
        }
        // switch view property
        if (this.state.project.autoSwitchView) {
            this.AddSwitchViewProperty(codegen_csharp_1.dotnet.Object);
        }
        // pipeline property
        this.add(new codegen_csharp_1.Property('Pipeline', exports_1.ClientRuntime.HttpPipeline, { getAccess: codegen_csharp_1.Access.Public, setAccess: codegen_csharp_1.Access.Public, description: `The instance of the <see cref="${exports_1.ClientRuntime.HttpPipeline}" /> that the remote call will use.` }));
        // client API property (gs01: fill this in correctly)
        const clientAPI = new codegen_csharp_1.ClassType(this.state.model.language.csharp?.namespace, this.state.model.language.csharp?.name || '');
        this.add(new codegen_csharp_1.LambdaProperty('Client', clientAPI, new codegen_csharp_1.LiteralExpression(`${this.state.project.serviceNamespace.moduleClass.declaration}.Instance.ClientAPI`), { description: 'The reference to the client API class.' }));
        this.add(new codegen_csharp_1.Method('StopProcessing', codegen_csharp_1.dotnet.Void, { access: codegen_csharp_1.Access.Protected, override: codegen_csharp_1.Modifier.Override, description: 'Interrupts currently running code within the command.' })).add(function* () {
            yield `((${exports_1.ClientRuntime.IEventListener})this).Cancel();`;
            yield 'base.StopProcessing();';
        });
        const $this = this;
        this.add(new codegen_csharp_1.Method('EndProcessing', codegen_csharp_1.dotnet.Void, { access: codegen_csharp_1.Access.Protected, override: codegen_csharp_1.Modifier.Override, description: 'Performs clean-up after the command execution' })).add(function* () {
            // gs01: remember what you were doing here to make it so these can be parallelized...
            if ($this.state.project.autoSwitchView) {
                yield $this.FlushResponse();
            }
            if (!$this.state.project.azure) {
                yield $this.eventListener.syncSignal(powershell_declarations_1.Events.CmdletEndProcessing);
            }
        });
        // debugging
        const brk = this.add(new codegen_csharp_1.Property('Break', powershell_declarations_1.SwitchParameter, { attributes: [], description: 'Wait for .NET debugger to attach' }));
        brk.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'DontShow = true', 'HelpMessage = "Wait for .NET debugger to attach"'] }));
        brk.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Runtime`] }));
        // Cmdlet Parameters for pipeline manipulations.
        const prepend = this.add(new codegen_csharp_1.Property('HttpPipelinePrepend', exports_1.ClientRuntime.SendAsyncSteps, { attributes: [], description: 'SendAsync Pipeline Steps to be prepended to the front of the pipeline' }));
        prepend.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'DontShow = true', 'HelpMessage = "SendAsync Pipeline Steps to be prepended to the front of the pipeline"'] }));
        prepend.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ValidateNotNull));
        prepend.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Runtime`] }));
        const append = this.add(new codegen_csharp_1.Property('HttpPipelineAppend', exports_1.ClientRuntime.SendAsyncSteps, { attributes: [], description: 'SendAsync Pipeline Steps to be appended to the front of the pipeline' }));
        append.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'DontShow = true', 'HelpMessage = "SendAsync Pipeline Steps to be appended to the front of the pipeline"'] }));
        append.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ValidateNotNull));
        append.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Runtime`] }));
        const proxyCredential = this.add(new codegen_csharp_1.Property('ProxyCredential', powershell_declarations_1.PSCredential, { attributes: [], description: 'Credentials for a proxy server to use for the remote call' }));
        proxyCredential.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'DontShow = true', 'HelpMessage = "Credentials for a proxy server to use for the remote call"'] }));
        proxyCredential.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ValidateNotNull));
        proxyCredential.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Runtime`] }));
        const useDefaultCreds = this.add(new codegen_csharp_1.Property('ProxyUseDefaultCredentials ', powershell_declarations_1.SwitchParameter, { attributes: [], description: 'Use the default credentials for the proxy' }));
        useDefaultCreds.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'DontShow = true', 'HelpMessage = "Use the default credentials for the proxy"'] }));
        useDefaultCreds.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Runtime`] }));
        const proxyUri = this.add(new codegen_csharp_1.Property('Proxy', codegen_csharp_1.System.Uri, { attributes: [], description: 'The URI for the proxy server to use' }));
        proxyUri.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'DontShow = true', 'HelpMessage = "The URI for the proxy server to use"'] }));
        proxyUri.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Runtime`] }));
        if (this.state.project.azure) {
            this.defaultProfile = this.add(new codegen_csharp_1.Property('DefaultProfile', powershell_declarations_1.PSObject, { description: 'The DefaultProfile parameter is not functional. Use the SubscriptionId parameter when available if executing the cmdlet against a different subscription' }));
            this.defaultProfile.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'HelpMessage = "The DefaultProfile parameter is not functional. Use the SubscriptionId parameter when available if executing the cmdlet against a different subscription."'] }));
            this.defaultProfile.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ValidateNotNull));
            this.defaultProfile.add(new codegen_csharp_1.Attribute(powershell_declarations_1.Alias, { parameters: ['"AzureRMContext"', '"AzureCredential"'] }));
            this.defaultProfile.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Azure`] }));
        }
    }
    NewImplementProcessRecord() {
        const $this = this;
        const operation = $this.operation;
        this.add(new codegen_csharp_1.Method('ProcessRecord', undefined, { access: codegen_csharp_1.Access.Protected, override: codegen_csharp_1.Modifier.Override, description: 'Performs execution of the command.' })).add(function* () {
            yield $this.eventListener.syncSignal(powershell_declarations_1.Events.CmdletProcessRecordStart);
            if ($this.state.project.azure) {
                yield $this.processRecordId.assign('System.Guid.NewGuid().ToString()');
            }
            yield (0, codegen_csharp_1.Try)(function* () {
                yield '// work';
                const normal = new codegen_csharp_1.Statements(function* () {
                    const acr = new codegen_csharp_1.LocalVariable('asyncCommandRuntime', codegen_csharp_1.dotnet.Var, { initializer: powershell_declarations_1.AsyncCommandRuntime.new(codegen_csharp_1.dotnet.This, $this.cancellationToken) });
                    yield (0, codegen_csharp_1.Using)(acr.declarationExpression, function* () {
                        yield `${acr}.Wait( ProcessRecordAsync(),${$this.cancellationToken});`;
                    });
                });
                if (operation.asjob) {
                    const asjob = $this.add(new codegen_csharp_1.Property('AsJob', powershell_declarations_1.SwitchParameter, { description: 'when specified, runs this cmdlet as a PowerShell job' }));
                    asjob.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'HelpMessage = "Run the command as a job"'] }));
                    asjob.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Runtime`] }));
                    const nowait = $this.add(new codegen_csharp_1.Property('NoWait', powershell_declarations_1.SwitchParameter, { description: 'when specified, will make the remote call, and return an AsyncOperationResponse, letting the remote operation continue asynchronously.' }));
                    nowait.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'HelpMessage = "Run the command asynchronously"'] }));
                    nowait.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Runtime`] }));
                }
                const work = operation.asjob ? function* () {
                    yield (0, codegen_csharp_1.If)('true == MyInvocation?.BoundParameters?.ContainsKey("AsJob")', function* () {
                        // clone the cmdlet instance, since the instance can be reused and overwrite data.
                        const instance = new codegen_csharp_1.LocalVariable('instance', codegen_csharp_1.dotnet.Var, { initializer: 'this.Clone()' });
                        yield instance.declarationStatement;
                        // create the job (which will set the CommandRuntime of the clone to the AsyncJob itself)
                        const job = new codegen_csharp_1.LocalVariable('job', codegen_csharp_1.dotnet.Var, { initializer: powershell_declarations_1.AsyncJob.new(instance, 'this.MyInvocation.Line, this.MyInvocation.MyCommand.Name, this._cancellationTokenSource.Token', 'this._cancellationTokenSource.Cancel') });
                        yield job.declarationStatement;
                        // add the job to the repository
                        yield `JobRepository.Add(${job});`;
                        // invoke the cmdlet's PRA
                        const task = new codegen_csharp_1.LocalVariable('task', codegen_csharp_1.dotnet.Var, { initializer: `${instance}.ProcessRecordAsync()` });
                        yield task.declarationStatement;
                        // have the AsyncJob monitor the lifetime of the Task
                        yield `${job}.Monitor(${task});`;
                        // return the job to the user now.
                        yield `WriteObject(${job});`;
                    });
                    yield (0, codegen_csharp_1.Else)(normal);
                } : normal;
                if ((0, command_operation_1.isWritableCmdlet)(operation) && !operation.details.csharp.supportShouldProcess) {
                    yield (0, codegen_csharp_1.If)(`ShouldProcess($"Call remote '${$this.apiCall.language.csharp?.name}' operation")`, work);
                }
                else {
                    yield work;
                }
            });
            const aggregateException = new codegen_csharp_1.Parameter('aggregateException', codegen_csharp_1.System.AggregateException);
            yield (0, codegen_csharp_1.Catch)(aggregateException, function* () {
                yield '// unroll the inner exceptions to get the root cause';
                yield (0, codegen_csharp_1.ForEach)('innerException', new codegen_csharp_1.LiteralExpression(`${aggregateException.use}.Flatten().InnerExceptions`), function* () {
                    yield $this.eventListener.syncSignal(powershell_declarations_1.Events.CmdletException, new codegen_csharp_1.LiteralExpression('$"{innerException.GetType().Name} - {innerException.Message} : {innerException.StackTrace}"'));
                    yield '// Write exception out to error channel.';
                    yield `WriteError( new ${powershell_declarations_1.ErrorRecord}(innerException,string.Empty, ${(0, powershell_declarations_1.ErrorCategory)('NotSpecified')}, null) );`;
                });
            });
            const exception = new codegen_csharp_1.Parameter('exception', codegen_csharp_1.System.Exception);
            yield (0, codegen_csharp_1.Catch)(exception, function* () {
                yield $this.eventListener.syncSignal(powershell_declarations_1.Events.CmdletException, new codegen_csharp_1.LiteralExpression(`$"{${exception.use}.GetType().Name} - {${exception.use}.Message} : {${exception.use}.StackTrace}"`));
                yield '// Write exception out to error channel.';
                yield `WriteError( new ${powershell_declarations_1.ErrorRecord}(${exception.use},string.Empty, ${(0, powershell_declarations_1.ErrorCategory)('NotSpecified')}, null) );`;
            }, { when: new codegen_csharp_1.LiteralExpression('(exception as System.Management.Automation.PipelineStoppedException)== null || (exception as System.Management.Automation.PipelineStoppedException).InnerException != null') });
            yield (0, codegen_csharp_1.Finally)(function* () {
                yield $this.eventListener.syncSignalNoCheck(powershell_declarations_1.Events.CmdletProcessRecordEnd);
            });
        });
    }
    NewImplementProcessRecordAsync() {
        const $this = this;
        const operationParameters = $this.operationParameters;
        const pipeline = $this.$('Pipeline');
        this.serializationMode = this.bodyParameter ? (this.operation.operationType === command_operation_1.OperationType.Create ? exports_1.ClientRuntime.SerializationMode.IncludeCreate : (this.operation.operationType === command_operation_1.OperationType.Update ? exports_1.ClientRuntime.SerializationMode.IncludeUpdate : undefined)) : undefined;
        const PRA = this.add(new codegen_csharp_1.Method('ProcessRecordAsync', codegen_csharp_1.System.Threading.Tasks.Task(), {
            access: codegen_csharp_1.Access.Protected, async: codegen_csharp_1.Modifier.Async,
            description: 'Performs execution of the command, working asynchronously if required.',
            returnsDescription: `A <see cref="${codegen_csharp_1.System.Threading.Tasks.Task()}" /> that will be complete when handling of the method is completed.`
        }));
        // we don't want to use SynchContext here.
        PRA.push((0, codegen_csharp_1.Using)('NoSynchronizationContext', ''));
        PRA.add(function* () {
            if ($this.apProp && $this.bodyParameter && $this.bodyParameterInfo) {
                // yield `${ClientRuntime}.DictionaryExtensions.HashTableToDictionary<${$this.bodyParameterInfo.type.declaration},${$this.bodyParameterInfo.valueType.declaration}>(${$this.apProp.value},${$this.bodyParameter.Cast($this.bodyParameterInfo.type)});`;
                let vt = $this.bodyParameterInfo.valueType.declaration;
                if (vt.endsWith('SwitchParameter')) {
                    vt = 'bool';
                }
                yield `${exports_1.ClientRuntime}.DictionaryExtensions.HashTableToDictionary<${vt}>(${$this.apProp.value},${$this.bodyParameter}.AdditionalProperties);`;
            }
            // construct the call to the operation
            if (!$this.state.project.azure) {
                yield $this.eventListener.signal(powershell_declarations_1.Events.CmdletProcessRecordAsyncStart);
            }
            yield $this.eventListener.signal(powershell_declarations_1.Events.CmdletGetPipeline);
            if ($this.state.project.azure) {
                yield pipeline.assign(new codegen_csharp_1.LiteralExpression(`${$this.state.project.serviceNamespace.moduleClass.declaration}.Instance.CreatePipeline(${$this.invocationInfo}, ${$this.correlationId}, ${$this.processRecordId}, this.ParameterSetName, this.ExtensibleParameters)`));
            }
            else {
                yield pipeline.assign(new codegen_csharp_1.LiteralExpression(`${$this.state.project.serviceNamespace.moduleClass.declaration}.Instance.CreatePipeline(${$this.invocationInfo}, this.ParameterSetName, this.ExtensibleParameters)`));
            }
            yield (0, codegen_csharp_1.If)((0, codegen_csharp_1.IsNotNull)($this.$('HttpPipelinePrepend')), pipeline.invokeMethod('Prepend', (0, codegen_csharp_1.toExpression)(`(this.CommandRuntime as Microsoft.Rest.ClientRuntime.PowerShell.IAsyncCommandRuntimeExtensions)?.Wrap(${$this.$('HttpPipelinePrepend')}) ?? ${$this.$('HttpPipelinePrepend')}`)));
            yield (0, codegen_csharp_1.If)((0, codegen_csharp_1.IsNotNull)($this.$('HttpPipelineAppend')), pipeline.invokeMethod('Append', (0, codegen_csharp_1.toExpression)(`(this.CommandRuntime as Microsoft.Rest.ClientRuntime.PowerShell.IAsyncCommandRuntimeExtensions)?.Wrap(${$this.$('HttpPipelineAppend')}) ?? ${$this.$('HttpPipelineAppend')}`)));
            yield '// get the client instance';
            // Add input pipeline if it is configured by developers.
            if ($this.operation.extensions.inputPipe) {
                const handlers = $this.operation.extensions.inputPipe;
                // default priority is 100
                handlers.sort((a, b) => { return (a.priority || 100) - (b.priority || 100); });
                for (const { index, handler } of handlers.map((handler, index) => ({ index, handler }))) {
                    yield `var handler_${index} = new ${handler.name}();`;
                    if (index > 0) {
                        yield `handler_${index - 1}.SetNextHandler(handler_${index});`;
                    }
                }
                yield 'handler_0.Process(this);';
            }
            const vps = $this.operation.details.csharp.virtualParameters || {
                body: [],
                operation: [],
            };
            for (const vParam of [...vps.body, ...vps.operation]) {
                if (vParam.hidden) {
                    const td = $this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(vParam.schema, true, $this.state, true);
                    yield (0, codegen_csharp_1.If)(`true == this.MyInvocation?.BoundParameters?.ContainsKey("${vParam.name}")`, `${vParam.name} = (${td.declaration})this.MyInvocation.BoundParameters["${vParam.name}"];`);
                }
            }
            // is there a body parameter we should include?
            // if ($this.bodyParameter) {
            //   operationParameters.push({ name: 'body', expression: $this.bodyParameter, isPathParam: false });
            // }
            yield (0, codegen_csharp_1.Try)(function* () {
                // make the call.
                let preProcess;
                switch ($this.operation.commandType) {
                    case command_operation_1.CommandType.GetPut:
                        preProcess = $this.GetPutPreProcess;
                        break;
                    case command_operation_1.CommandType.Atomic:
                    default:
                        preProcess = undefined;
                        break;
                }
                const actualCall = function* () {
                    yield $this.eventListener.signal(powershell_declarations_1.Events.CmdletBeforeAPICall);
                    yield $this.ImplementCall(preProcess);
                    yield $this.eventListener.signal(powershell_declarations_1.Events.CmdletAfterAPICall);
                };
                if ($this.state.project.azure && operationParameters.find(each => each.expression && each.expression.value === 'SubscriptionId') && $this.operation.details.csharp.verb.toLowerCase() === 'get') {
                    yield 'foreach( var SubscriptionId in this.SubscriptionId )';
                    yield (0, codegen_csharp_1.BlockStatement)(actualCall);
                }
                else {
                    yield actualCall;
                }
            });
            const ure = new codegen_csharp_1.Parameter('urexception', { declaration: `${exports_1.ClientRuntime.fullName}.UndeclaredResponseException` });
            yield (0, codegen_csharp_1.Catch)(ure, function* () {
                yield `WriteError(new global::System.Management.Automation.ErrorRecord(${ure.value}, ${ure.value}.StatusCode.ToString(), global::System.Management.Automation.ErrorCategory.InvalidOperation, new { ${operationParameters.filter(e => (0, codegen_csharp_1.valueOf)(e.expression) !== 'null').map(each => `${each.name}=${each.expression}`).join(',')}})
{
  ErrorDetails = new global::System.Management.Automation.ErrorDetails(${ure.value}.Message) { RecommendedAction = ${ure.value}.Action }
});`;
            });
            yield (0, codegen_csharp_1.Finally)(function* () {
                yield $this.eventListener.signalNoCheck(powershell_declarations_1.Events.CmdletProcessRecordAsyncEnd);
            });
        });
    }
    *ImplementCall(preProcess) {
        const $this = this;
        const operation = $this.operation;
        const apiCall = $this.apiCall;
        const operationParameters = $this.operationParameters;
        const callbackMethods = $this.callbackMethods;
        const pipeline = $this.$('Pipeline');
        const serializationMode = $this.serializationMode;
        const bodyParameter = this.bodyParameter ? { name: 'body', value: (0, codegen_csharp_1.valueOf)(this.bodyParameter) } : undefined;
        const pathParamsNotInIdentity = [];
        const pathParamsInIdentity = [];
        const pathParamsInIdentitySerializedName = [];
        const headerParams = [];
        const queryParams = [];
        const otherParams = [];
        const idschema = (0, linq_1.values)($this.state.project.model.schemas.objects).first(each => each.language.default.uid === 'universal-parameter-type');
        let httpOperationName = `${apiCall.language.csharp?.name}`;
        if (idschema) {
            const allVPs = (0, schema_1.getAllPublicVirtualProperties)(idschema.language.csharp?.virtualProperties);
            const props = [...(0, linq_1.values)(idschema.properties)];
            operationParameters.forEach(each => {
                const pascalName = (0, codegen_1.pascalCase)(`${each.name}`);
                //push parameters that is not path parameters into allParams and idOpParamsNotFromIdentity
                if (each.parameterLocation !== codemodel_v3_1.ParameterLocation.Path) {
                    const param = {
                        name: undefined,
                        value: (0, codegen_csharp_1.valueOf)(each.expression)
                    };
                    switch (each.parameterLocation) {
                        case codemodel_v3_1.ParameterLocation.Header:
                            headerParams.push(param);
                            break;
                        case codemodel_v3_1.ParameterLocation.Query:
                            queryParams.push(param);
                            break;
                        default:
                            otherParams.push(param);
                            break;
                    }
                    return;
                }
                else {
                    const match = props.find(p => (0, codegen_1.pascalCase)(p.serializedName) === pascalName);
                    if (match) {
                        const defaultOfType = $this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(match.schema, true, $this.state).defaultOfType;
                        // match up vp name
                        const vp = allVPs.find(pp => (0, codegen_1.pascalCase)(pp.property.serializedName) === pascalName);
                        //push path parameters that form current identity into allParams, idOpParamsFromIdentity and idOpParamsFromIdentityserializedName
                        if (vp && each.expression === codegen_csharp_1.dotnet.Null) {
                            const param = {
                                name: `${$this.inputObjectParameterName}.${vp.name}`,
                                value: `${$this.inputObjectParameterName}.${vp.name} ?? ${defaultOfType}`
                            };
                            pathParamsInIdentity.push(param);
                            pathParamsInIdentitySerializedName.push(match.serializedName);
                            return;
                        }
                        // fall back!
                        this.state.debug(`Unable to match identity parameter '${each.name}' member to appropriate virtual parameter. (Guessing '${(0, codegen_1.pascalCase)(match.language.csharp?.name ?? '')}').`, {});
                        //push path parameters that current identity does not contain into allParams and idOpParamsNotFromIdentity
                        const param = {
                            name: `${(0, codegen_1.pascalCase)(match.language.csharp?.name ?? '')}`,
                            value: `${(0, codegen_1.pascalCase)(match.language.csharp?.name ?? '')}`
                        };
                        pathParamsNotInIdentity.push(param);
                    }
                    else {
                        this.state.debug(`Unable to match identity parameter '${each.name}' member to appropriate virtual parameter. (Guessing '${pascalName}')`, {});
                        /*
                          push path parameters do not match the name in identity schema into allParams and idOpParamsNotFromIdentity
                          for example, module 'Service' has only one GET API:
                            /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Service/resource1/{resource1Name}/resource2/{resource2Name}/resource3/{resource3Name}
                            the identity schema 'ServiceIdentity.cs' will have properties: {subscriptionId, resourceGroupName, resource1Name, resource2Name, resource3Name}
                          for variant which has (identity of resource2 + resource3Name) combined as parameters, the parameter name for 'resource3Name' is called 'Name' which do not match 'resource3Name' in 'ServiceIdentity.cs'
                        */
                        const param = {
                            name: `${pascalName}`,
                            value: `${pascalName}`
                        };
                        pathParamsNotInIdentity.push(param);
                    }
                }
            });
        }
        if ($this.isViaIdentity) {
            //when identity does not contain property: 'Id'
            const identityFromPathParams = function* () {
                yield '// try to call with PATH parameters from Input Object';
                if (idschema) {
                    for (const opParam of pathParamsInIdentity) {
                        if (opParam && opParam.name) {
                            yield (0, codegen_csharp_1.If)((0, codegen_csharp_1.IsNull)(opParam.name), `ThrowTerminatingError( new ${powershell_declarations_1.ErrorRecord}(new global::System.Exception("${$this.inputObjectParameterName} has null value for ${opParam.name}"),string.Empty, ${(0, powershell_declarations_1.ErrorCategory)('InvalidArgument')}, ${$this.inputObjectParameterName}) );`);
                        }
                    }
                    const pathParameters = [...pathParamsInIdentity.map(each => (0, codegen_csharp_1.toExpression)(each.value)), ...pathParamsNotInIdentity.map(each => (0, codegen_csharp_1.toExpression)(each.value))];
                    const nonPathParameters = [...headerParams.map(each => (0, codegen_csharp_1.toExpression)(each.value)), ...queryParams.map(each => (0, codegen_csharp_1.toExpression)(each.value)), ...otherParams.map(each => (0, codegen_csharp_1.toExpression)(each.value))];
                    const parameters = bodyParameter ? [...pathParameters, ...nonPathParameters, (0, codegen_csharp_1.toExpression)(bodyParameter.value), ...callbackMethods, codegen_csharp_1.dotnet.This, pipeline] : [...pathParameters, ...nonPathParameters, ...callbackMethods, codegen_csharp_1.dotnet.This, pipeline];
                    if (serializationMode) {
                        parameters.push(serializationMode);
                    }
                    if (preProcess) {
                        yield preProcess($this, pathParameters, [...otherParams.map(each => (0, codegen_csharp_1.toExpression)(each.value)), codegen_csharp_1.dotnet.This, pipeline], false);
                    }
                    yield `await this.${$this.$('Client').invokeMethod(httpOperationName, ...parameters).implementation}`;
                }
            };
            //when identity does contain property: 'Id'
            const identityParams = function* () {
                const path = apiCall.requests?.[0].protocol.http?.path;
                let pathParams = '';
                //append path parameters which are not part of current identity
                pathParamsNotInIdentity.forEach(each => {
                    const serializedName = (0, linq_1.values)($this.properties)
                        .where(p => p.metadata.parameterDefinition)
                        .first(p => p.name === each.name)?.metadata.parameterDefinition.language.csharp.serializedName;
                    if (each.name && serializedName) {
                        if (!pathParams) {
                            pathParams += '$"';
                        }
                        const resourceName = (0, resourceName_1.getResourceNameFromPath)(path, serializedName);
                        if (resourceName) {
                            pathParams += `/${resourceName}/{(global::System.Uri.EscapeDataString(this.${each.name}.ToString()))}`;
                        }
                    }
                });
                //add child resource to path for list operation
                if ($this.operation.variant.startsWith('List') && $this.operation.callGraph[0].requests?.[0].protocol.http?.method.toLowerCase() !== 'post' && pathParamsInIdentitySerializedName?.[pathParamsInIdentitySerializedName.length - 1]) {
                    const childResourceName = (0, resourceName_1.getChildResourceNameFromPath)(path, pathParamsInIdentitySerializedName?.[pathParamsInIdentitySerializedName.length - 1]);
                    if (pathParams && pathParams.length > 0) {
                        pathParams += `/${childResourceName}`;
                    }
                    else {
                        pathParams += `"/${childResourceName}`;
                    }
                }
                if (pathParams && pathParams.length > 0) {
                    pathParams += '";';
                    yield `this.${$this.inputObjectParameterName}.Id += ${pathParams}`;
                }
                const pathParameters = [(0, codegen_csharp_1.toExpression)(`${$this.inputObjectParameterName}.Id`)];
                const nonPathParameters = [...headerParams.map(each => (0, codegen_csharp_1.toExpression)(each.value)), ...queryParams.map(each => (0, codegen_csharp_1.toExpression)(each.value)), ...otherParams.map(each => (0, codegen_csharp_1.toExpression)(each.value))];
                const parameters = bodyParameter ? [...pathParameters, ...nonPathParameters, (0, codegen_csharp_1.toExpression)(bodyParameter.value), ...callbackMethods, codegen_csharp_1.dotnet.This, pipeline] : [...pathParameters, ...nonPathParameters, ...callbackMethods, codegen_csharp_1.dotnet.This, pipeline];
                if (serializationMode) {
                    parameters.push(serializationMode);
                }
                if (preProcess) {
                    yield preProcess($this, pathParameters, [...otherParams.map(each => (0, codegen_csharp_1.toExpression)(each.value)), codegen_csharp_1.dotnet.This, pipeline], true);
                }
                yield `await this.${$this.$('Client').invokeMethod(`${httpOperationName}ViaIdentity`, ...parameters).implementation}`;
            };
            if (idschema && (0, linq_1.values)(idschema.properties).first(each => each.language.csharp?.uid === 'universal-parameter:resource identity')) {
                yield (0, codegen_csharp_1.If)(`${$this.inputObjectParameterName}?.Id != null`, identityParams);
                yield (0, codegen_csharp_1.Else)(identityFromPathParams);
            }
            else {
                yield identityFromPathParams;
            }
        }
        else {
            const pathParameters = [...pathParamsInIdentity.map(each => (0, codegen_csharp_1.toExpression)(each.value)), ...pathParamsNotInIdentity.map(each => (0, codegen_csharp_1.toExpression)(each.value))];
            const nonPathParameters = [...headerParams.map(each => (0, codegen_csharp_1.toExpression)(each.value)), ...queryParams.map(each => (0, codegen_csharp_1.toExpression)(each.value)), ...otherParams.map(each => (0, codegen_csharp_1.toExpression)(each.value))];
            let parameters = bodyParameter ? [...pathParameters, ...nonPathParameters, (0, codegen_csharp_1.toExpression)(bodyParameter.value), ...callbackMethods, codegen_csharp_1.dotnet.This, pipeline] : [...pathParameters, ...nonPathParameters, ...callbackMethods, codegen_csharp_1.dotnet.This, pipeline];
            if (serializationMode) {
                parameters.push(serializationMode);
            }
            if (operation.variant.includes('ViaJsonString') || operation.variant.includes('ViaJsonFilePath')) {
                httpOperationName = `${httpOperationName}ViaJsonString`;
                const jsonParameter = new codegen_csharp_1.Field('_jsonString', codegen_csharp_1.System.String);
                parameters = [...pathParameters, ...nonPathParameters, jsonParameter, ...callbackMethods, codegen_csharp_1.dotnet.This, pipeline];
            }
            if (preProcess) {
                yield preProcess($this, pathParameters, [...otherParams.map(each => (0, codegen_csharp_1.toExpression)(each.value)), codegen_csharp_1.dotnet.This, pipeline], false);
            }
            yield `await this.${$this.$('Client').invokeMethod(httpOperationName, ...parameters).implementation}`;
        }
    }
    GetPutPreProcess(cmdlet, pathParams, nonPathParams, viaIdentity) {
        const $this = cmdlet;
        const updateBodyMethod = new codegen_csharp_1.Method(`Update${$this.bodyParameter?.value}`, codegen_csharp_1.dotnet.Void, {
            access: codegen_csharp_1.Access.Private
        });
        const httpOperationName = `${$this.operation.callGraph[0].language.csharp?.name}${viaIdentity ? 'ViaIdentity' : ''}WithResult`;
        if (!$this.hasMethodWithSameDeclaration(updateBodyMethod)) {
            updateBodyMethod.add(function* () {
                const bodyParameters = $this.properties.filter(each => {
                    for (const attribute of each.attributes) {
                        for (const parameter of attribute.parameters) {
                            if ('global::Microsoft.Rest.ParameterCategory.Body' === (0, codegen_csharp_1.valueOf)(parameter)) {
                                return true;
                            }
                        }
                    }
                    return false;
                });
                for (const param of bodyParameters) {
                    yield (0, codegen_csharp_1.If)(`(bool)(true == this.MyInvocation?.BoundParameters.ContainsKey("${param.name}"))`, `this.${param.name} = (${param.type.declaration})(this.MyInvocation?.BoundParameters["${param.name}"]);`);
                }
            });
            $this.add(updateBodyMethod);
        }
        const getPut = function* () {
            yield `${$this.bodyParameter?.value} = await this.${$this.$('Client').invokeMethod(httpOperationName, ...[...pathParams, ...nonPathParams]).implementation}`;
            yield `this.${updateBodyMethod.name}();`;
        };
        return new codegen_csharp_1.Statements(getPut);
    }
    NewImplementResponseMethod() {
        const $this = this;
        const apiCall = $this.apiCall;
        const operationParameters = $this.operationParameters;
        const callbackMethods = $this.callbackMethods;
        const pipeline = $this.$('Pipeline');
        // make callback methods
        for (const each of (0, linq_1.values)($this.responses)) {
            const parameters = new Array();
            const isBinary = each.binary;
            parameters.push(new codegen_csharp_1.Parameter('responseMessage', codegen_csharp_1.System.Net.Http.HttpResponseMessage, { description: `the raw response message as an ${codegen_csharp_1.System.Net.Http.HttpResponseMessage}.` }));
            if (each.language.csharp?.responseType) {
                parameters.push(new codegen_csharp_1.Parameter('response', codegen_csharp_1.System.Threading.Tasks.Task({ declaration: each.language.csharp?.responseType }), {
                    description: `the body result as a <see cref="${each.language.csharp?.responseType.replace(/\[|\]|\?/g, '')}">${each.language.csharp?.responseType}</see> from the remote call`
                }));
            }
            if (each.language.csharp?.headerType) {
                parameters.push(new codegen_csharp_1.Parameter('headers', codegen_csharp_1.System.Threading.Tasks.Task({ declaration: each.language.csharp.headerType }), { description: `the header result as a <see cref="${each.language.csharp.headerType}" /> from the remote call` }));
            }
            if (isBinary) {
                parameters.push(new codegen_csharp_1.Parameter('response', codegen_csharp_1.System.Threading.Tasks.Task({ declaration: 'global::System.IO.Stream' }), { description: 'the body result as a <see cref="global::System.IO.Stream" /> from the remote call' }));
            }
            const override = `override${(0, codegen_1.pascalCase)(each.language.csharp?.name || '')}`;
            const returnNow = new codegen_csharp_1.Parameter('returnNow', codegen_csharp_1.System.Threading.Tasks.Task(codegen_csharp_1.dotnet.Bool), { modifier: codegen_csharp_2.ParameterModifier.Ref, description: `/// Determines if the rest of the ${each.language.csharp?.name} method should be processed, or if the method should return immediately (set to true to skip further processing )` });
            const overrideResponseMethod = new codegen_csharp_1.PartialMethod(override, codegen_csharp_1.dotnet.Void, {
                parameters: [...parameters, returnNow],
                description: `<c>${override}</c> will be called before the regular ${each.language.csharp?.name} has been processed, allowing customization of what happens on that response. Implement this method in a partial class to enable this behavior`,
                returnsDescription: `A <see cref="${codegen_csharp_1.System.Threading.Tasks.Task()}" /> that will be complete when handling of the method is completed.`
            });
            $this.add(overrideResponseMethod);
            const responseMethod = new codegen_csharp_1.Method(`${each.language.csharp?.name}`, codegen_csharp_1.System.Threading.Tasks.Task(), {
                access: codegen_csharp_1.Access.Private,
                parameters,
                async: codegen_csharp_1.Modifier.Async,
                description: each.language.csharp?.description,
                returnsDescription: `A <see cref="${codegen_csharp_1.System.Threading.Tasks.Task()}" /> that will be complete when handling of the method is completed.`
            });
            responseMethod.push((0, codegen_csharp_1.Using)('NoSynchronizationContext', ''));
            responseMethod.add(function* () {
                const skip = (0, codegen_csharp_2.Local)('_returnNow', `${codegen_csharp_1.System.Threading.Tasks.Task(codegen_csharp_1.dotnet.Bool).declaration}.FromResult(${codegen_csharp_1.dotnet.False})`);
                yield skip.declarationStatement;
                yield `${overrideResponseMethod.invoke(...parameters, `ref ${skip.value}`)};`;
                yield `// if ${override} has returned true, then return right away.`;
                yield (0, codegen_csharp_1.If)((0, codegen_csharp_1.And)((0, codegen_csharp_1.IsNotNull)(skip), `await ${skip}`), (0, codegen_csharp_1.Return)());
                if (each.language.csharp?.isErrorResponse) {
                    // this should write an error to the error channel.
                    yield `// Error Response : ${each.protocol.http?.statusCodes[0]}`;
                    const unexpected = function* () {
                        yield '// Unrecognized Response. Create an error record based on what we have.';
                        const ex = (each.language.csharp?.responseType) ?
                            (0, codegen_csharp_2.Local)('ex', `new ${exports_1.ClientRuntime.name}.RestException<${each.language.csharp.responseType}>(responseMessage, await response)`) :
                            (0, codegen_csharp_2.Local)('ex', `new ${exports_1.ClientRuntime.name}.RestException(responseMessage)`);
                        yield ex.declarationStatement;
                        yield `WriteError( new global::System.Management.Automation.ErrorRecord(${ex.value}, ${ex.value}.Code, global::System.Management.Automation.ErrorCategory.InvalidOperation, new { ${operationParameters.filter(e => (0, codegen_csharp_1.valueOf)(e.expression) !== 'null').map(each => `${each.name}=${each.expression}`).join(', ')} })
{
  ErrorDetails = new global::System.Management.Automation.ErrorDetails(${ex.value}.Message) { RecommendedAction = ${ex.value}.Action }
});`;
                    };
                    if (each.schema !== undefined) {
                        // the schema should be the error information.
                        // this supports both { error { message, code} } and { message, code}
                        let props = (0, schema_1.getAllPublicVirtualProperties)(each.schema.language.csharp?.virtualProperties);
                        const errorProperty = (0, linq_1.values)(props).first(p => p.property.serializedName === 'error');
                        let ep = '';
                        if (errorProperty) {
                            props = (0, schema_1.getAllPublicVirtualProperties)(errorProperty.property.schema.language.csharp?.virtualProperties);
                            ep = `${errorProperty.name}?.`;
                        }
                        const codeProp = props.find(p => p.name.toLowerCase().indexOf('code') > -1); // first property with 'code'
                        const messageProp = props.find(p => p.name.toLowerCase().indexOf('message') > -1); // first property with 'message'
                        const actionProp = props.find(p => p.name.toLowerCase().indexOf('action') > -1); // first property with 'action'
                        if (codeProp && messageProp) {
                            const lcode = new codegen_csharp_1.LocalVariable('code', codegen_csharp_1.dotnet.Var, { initializer: `(await response)?.${ep}${codeProp.name}` });
                            const lmessage = new codegen_csharp_1.LocalVariable('message', codegen_csharp_1.dotnet.Var, { initializer: `(await response)?.${ep}${messageProp.name}` });
                            const laction = actionProp ? new codegen_csharp_1.LocalVariable('action', codegen_csharp_1.dotnet.Var, { initializer: `(await response)?.${ep}${actionProp.name} ?? ${codegen_csharp_1.System.String.Empty}` }) : undefined;
                            yield lcode;
                            yield lmessage;
                            yield laction;
                            yield (0, codegen_csharp_1.If)((0, codegen_csharp_1.Or)((0, codegen_csharp_1.IsNull)(lcode), ((0, codegen_csharp_1.IsNull)(lmessage))), unexpected);
                            yield (0, codegen_csharp_1.Else)(`WriteError( new global::System.Management.Automation.ErrorRecord(new global::System.Exception($"[{${lcode}}] : {${lmessage}}"), ${lcode}?.ToString(), global::System.Management.Automation.ErrorCategory.InvalidOperation, new { ${operationParameters.filter(e => (0, codegen_csharp_1.valueOf)(e.expression) !== 'null').map(each => `${each.name}=${each.expression}`).join(', ')} })
{
  ErrorDetails = new global::System.Management.Automation.ErrorDetails(${lmessage}) { RecommendedAction = ${laction || codegen_csharp_1.System.String.Empty} }
});`);
                            return;
                        }
                        else {
                            yield unexpected;
                            return;
                        }
                    }
                    else {
                        yield unexpected;
                        return;
                    }
                }
                yield `// ${each.language.csharp?.name} - response for ${each.protocol.http?.statusCodes[0]} / ${(0, linq_1.values)(each.protocol.http?.mediaTypes).join('/')}`;
                if ('schema' in each) {
                    const schema = each.schema;
                    const props = (0, schema_1.getAllPublicVirtualProperties)(schema.language.csharp?.virtualProperties);
                    const rType = $this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(schema, true, $this.state);
                    const result = new codegen_csharp_1.LocalVariable('result', codegen_csharp_1.dotnet.Var, { initializer: new codegen_csharp_1.LiteralExpression('(await response)') });
                    yield `// (await response) // should be ${rType.declaration}`;
                    yield result.declarationStatement;
                    if (apiCall.language.csharp?.pageable) {
                        const pageable = apiCall.language.csharp.pageable;
                        if ($this.clientsidePagination) {
                            yield '// clientside pagination enabled';
                        }
                        yield '// response should be returning an array of some kind. +Pageable';
                        yield `// ${pageable.responseType} / ${pageable.itemName || '<none>'} / ${pageable.nextLinkName || '<none>'}`;
                        switch (pageable.responseType) {
                            // the result is (or works like a x-ms-pageable)
                            case 'pageable':
                            case 'nested-array':
                                {
                                    const valueProperty = schema.properties?.find(p => p.serializedName === pageable.itemName);
                                    const nextLinkProperty = schema?.properties?.find(p => p.serializedName === pageable.nextLinkName);
                                    if (valueProperty && nextLinkProperty) {
                                        // it's pageable!
                                        // write out the current contents
                                        const vp = (0, schema_1.getVirtualPropertyFromPropertyName)(schema.language.csharp?.virtualProperties, valueProperty.serializedName);
                                        if (vp) {
                                            if ($this.clientsidePagination) {
                                                yield ((0, codegen_csharp_1.If)('(ulong)result.Value.Count <= this.PagingParameters.Skip', function* () {
                                                    yield ('this.PagingParameters.Skip = this.PagingParameters.Skip - (ulong)result.Value.Count;');
                                                }));
                                                yield (0, codegen_csharp_1.Else)(function* () {
                                                    yield ('ulong toRead = Math.Min(this.PagingParameters.First, (ulong)result.Value.Count - this.PagingParameters.Skip);');
                                                    yield ('var requiredResult = result.Value.GetRange((int)this.PagingParameters.Skip, (int)toRead);');
                                                    yield $this.WriteObjectWithViewControl('requiredResult', true);
                                                    yield ('this.PagingParameters.Skip = 0;');
                                                    yield ('this.PagingParameters.First = this.PagingParameters.First <= toRead ? 0 : this.PagingParameters.First - toRead;');
                                                });
                                            }
                                            else {
                                                yield $this.WriteObjectWithViewControl(`${result.value}.${vp.name}`, true);
                                            }
                                        }
                                        const nl = (0, schema_1.getVirtualPropertyFromPropertyName)(schema.language.csharp?.virtualProperties, nextLinkProperty.serializedName);
                                        if (nl) {
                                            $this.add(new codegen_csharp_1.Field('_isFirst', codegen_csharp_1.dotnet.Bool, {
                                                access: codegen_csharp_1.Access.Private,
                                                initialValue: new codegen_csharp_1.LiteralExpression('true'),
                                                description: 'A flag to tell whether it is the first onOK call.'
                                            }));
                                            $this.add(new codegen_csharp_1.Field('_nextLink', codegen_csharp_1.dotnet.String, {
                                                access: codegen_csharp_1.Access.Private,
                                                description: 'Link to retrieve next page.'
                                            }));
                                            const nextLinkName = `${result.value}.${nl.name}`;
                                            yield `_nextLink = ${nextLinkName};`;
                                            const nextLinkCondition = $this.clientsidePagination ? '!String.IsNullOrEmpty(_nextLink) && this.PagingParameters.First > 0' : '!String.IsNullOrEmpty(_nextLink)';
                                            yield ((0, codegen_csharp_1.If)('_isFirst', function* () {
                                                yield '_isFirst = false;';
                                                yield ((0, codegen_csharp_1.While)(nextLinkCondition, (0, codegen_csharp_1.If)('responseMessage.RequestMessage is System.Net.Http.HttpRequestMessage requestMessage ', function* () {
                                                    yield `requestMessage = requestMessage.Clone(new global::System.Uri( _nextLink ),${exports_1.ClientRuntime.Method.Get} );`;
                                                    yield $this.eventListener.signal(powershell_declarations_1.Events.FollowingNextLink);
                                                    yield `await this.${$this.$('Client').invokeMethod(`${apiCall.language.csharp?.name}_Call`, ...[(0, codegen_csharp_1.toExpression)('requestMessage'), ...callbackMethods, codegen_csharp_1.dotnet.This, pipeline]).implementation}`;
                                                })));
                                            }));
                                        }
                                        return;
                                    }
                                    else if (valueProperty) {
                                        // it's just a nested array
                                        const p = (0, codemodel_v3_1.getVirtualPropertyFromPropertyName)(schema.language.csharp?.virtualProperties, valueProperty.serializedName);
                                        if (p) {
                                            yield $this.WriteObjectWithViewControl(`${result.value}.${p.name}`, true);
                                        }
                                        return;
                                    }
                                }
                                break;
                            // it's just an array,
                            case 'array':
                                // just write-object(enumerate) with the output
                                yield $this.WriteObjectWithViewControl(result.value, true);
                                return;
                        }
                        // ok, let's see if the response type
                    }
                    // we expect to get back some data from this call.
                    if ($this.hasStreamOutput && $this.outFileParameter) {
                        const outfile = $this.outFileParameter;
                        const provider = (0, codegen_csharp_2.Local)('provider');
                        provider.initializer = undefined;
                        const paths = (0, codegen_csharp_2.Local)('paths', `this.SessionState.Path.GetResolvedProviderPathFromPSPath(${outfile.value}, out ${provider.declarationExpression})`);
                        yield paths.declarationStatement;
                        yield (0, codegen_csharp_1.If)(`${provider.value}.Name != "FileSystem" || ${paths.value}.Count == 0`, `ThrowTerminatingError( new System.Management.Automation.ErrorRecord(new global::System.Exception("Invalid output path."),string.Empty, global::System.Management.Automation.ErrorCategory.InvalidArgument, ${outfile.value}) );`);
                        yield (0, codegen_csharp_1.If)(`${paths.value}.Count > 1`, `ThrowTerminatingError( new System.Management.Automation.ErrorRecord(new global::System.Exception("Multiple output paths not allowed."),string.Empty, global::System.Management.Automation.ErrorCategory.InvalidArgument, ${outfile.value}) );`);
                        if (rType.declaration === codegen_csharp_1.System.IO.Stream.declaration) {
                            // this is a stream output. write to outfile
                            const stream = (0, codegen_csharp_2.Local)('stream', result.value);
                            yield (0, codegen_csharp_1.Using)(stream.declarationExpression, function* () {
                                const fileStream = (0, codegen_csharp_2.Local)('fileStream', `global::System.IO.File.OpenWrite(${paths.value}[0])`);
                                yield (0, codegen_csharp_1.Using)(fileStream.declarationExpression, `await ${stream.value}.CopyToAsync(${fileStream.value});`);
                            });
                        }
                        else {
                            // assuming byte array output (via result)
                            yield `global::System.IO.File.WriteAllBytes(${paths.value}[0],${result.value});`;
                        }
                        yield (0, codegen_csharp_1.If)('true == MyInvocation?.BoundParameters?.ContainsKey("PassThru")', function* () {
                            // no return type. Let's just return ... true?
                            yield 'WriteObject(true);';
                        });
                        return;
                    }
                    //  let's just return the result object (or unwrapped result object)
                    yield $this.WriteObjectWithViewControl(result.value);
                    return;
                }
                // in m4, there will be no schema deinfed for the binary response, instead, we will have a field called binary with value true.
                if ('binary' in each) {
                    yield '// (await response) // should be global::System.IO.Stream';
                    if ($this.hasStreamOutput && $this.outFileParameter) {
                        const outfile = $this.outFileParameter;
                        const provider = (0, codegen_csharp_2.Local)('provider');
                        provider.initializer = undefined;
                        const paths = (0, codegen_csharp_2.Local)('paths', 'new global::System.Collections.ObjectModel.Collection<global::System.String>()');
                        yield paths.declarationStatement;
                        yield (0, codegen_csharp_1.Try)(function* () {
                            yield `${paths.value} = this.SessionState.Path.GetResolvedProviderPathFromPSPath(${outfile.value}, out ${provider.declarationExpression});`;
                            yield (0, codegen_csharp_1.If)(`${provider.value}.Name != "FileSystem" || ${paths.value}.Count == 0`, `ThrowTerminatingError(new System.Management.Automation.ErrorRecord(new global::System.Exception("Invalid output path."), string.Empty, global::System.Management.Automation.ErrorCategory.InvalidArgument, ${outfile.value}));`);
                            yield (0, codegen_csharp_1.If)(`${paths.value}.Count > 1`, `ThrowTerminatingError(new System.Management.Automation.ErrorRecord(new global::System.Exception("Multiple output paths not allowed."), string.Empty, global::System.Management.Automation.ErrorCategory.InvalidArgument, ${outfile.value}));`);
                        });
                        const notfound = new codegen_csharp_1.Parameter('', { declaration: 'global::System.Management.Automation.ItemNotFoundException' });
                        yield (0, codegen_csharp_1.Catch)(notfound, function* () {
                            yield '// If the file does not exist, we will try to create it';
                            yield `${paths.value}.Add(${outfile.value});`;
                        });
                        // this is a stream output. write to outfile
                        const stream = (0, codegen_csharp_2.Local)('stream', 'await response');
                        yield (0, codegen_csharp_1.Using)(stream.declarationExpression, function* () {
                            const fileStream = (0, codegen_csharp_2.Local)('fileStream', `global::System.IO.File.OpenWrite(${paths.value}[0])`);
                            yield (0, codegen_csharp_1.Using)(fileStream.declarationExpression, `await ${stream.value}.CopyToAsync(${fileStream.value});`);
                        });
                        yield (0, codegen_csharp_1.If)('true == MyInvocation?.BoundParameters?.ContainsKey("PassThru")', function* () {
                            // no return type. Let's just return ... true?
                            yield 'WriteObject(true);';
                        });
                        return;
                    }
                }
                yield (0, codegen_csharp_1.If)('true == MyInvocation?.BoundParameters?.ContainsKey("PassThru")', function* () {
                    // no return type. Let's just return ... true?
                    yield 'WriteObject(true);';
                });
            });
            $this.add(responseMethod);
        }
    }
    NewImplementSerialization(operation) {
        const $this = this;
        // clone
        if (operation.asjob) {
            const clone = this.add(new codegen_csharp_1.Method('Clone', this, {
                description: 'Creates a duplicate instance of this cmdlet (via JSON serialization).',
                returnsDescription: `a duplicate instance of ${this.name}`,
            }));
            clone.add(function* () {
                const i = new codegen_csharp_1.LocalVariable('clone', codegen_csharp_1.dotnet.Var, {
                    initializer: $this.new()
                });
                yield i.declarationStatement;
                if ($this.state.project.azure) {
                    for (const f of [$this.correlationId, $this.processRecordId, $this.defaultProfile]) {
                        yield `${i.value}.${f} = this.${f};`;
                    }
                }
                for (const f of [$this.invocationInfo, 'Proxy', 'Pipeline', 'AsJob', 'Break', 'ProxyCredential', 'ProxyUseDefaultCredentials', 'HttpPipelinePrepend', 'HttpPipelineAppend',]) {
                    yield `${i.value}.${f} = this.${f};`;
                }
                for (const f of $this.thingsToSerialize) {
                    yield `${i.value}.${f} = this.${f};`;
                }
                // _name = this._name,
                //_parametersBody = this._parametersBody,
                //_resourceGroupName = this._resourceGroupName,
                //_subscriptionId = this._subscriptionId,
                yield (0, codegen_csharp_1.Return)(i);
            });
        }
    }
    implementConstructors() {
        // default constructor
        this.add(new codegen_csharp_1.Constructor(this, { description: `Initializes a new instance of the <see cref="${this.name}" /> cmdlet class.` }));
    }
    implementIContext() {
        const extensibleParameters = this.add(new codegen_csharp_1.Field('_extensibleParameters', codegen_csharp_1.System.Collections.Generic.Dictionary(codegen_csharp_1.System.String, codegen_csharp_1.System.Object), {
            access: codegen_csharp_1.Access.Private,
            initialValue: new codegen_csharp_1.LiteralExpression('new System.Collections.Generic.Dictionary<string, object>()'),
            description: 'A dictionary to carry over additional data for pipeline.'
        }));
        const extensibleParametersProp = new codegen_csharp_1.Property('ExtensibleParameters', codegen_csharp_1.System.Collections.Generic.IDictionary(codegen_csharp_1.System.String, codegen_csharp_1.System.Object), { description: 'Accessor for extensibleParameters.' });
        extensibleParametersProp.get = (0, codegen_csharp_1.toExpression)(`${extensibleParameters.value} `);
        this.add(extensibleParametersProp);
    }
    implementIEventListener() {
        const $this = this;
        const cts = this.add(new codegen_csharp_1.Field('_cancellationTokenSource', codegen_csharp_1.System.Threading.CancellationTokenSource, {
            access: codegen_csharp_1.Access.Private,
            initialValue: new codegen_csharp_1.LiteralExpression(`new ${codegen_csharp_1.System.Threading.CancellationTokenSource.declaration}()`),
            description: `The <see cref="${codegen_csharp_1.System.Threading.CancellationTokenSource}" /> for this operation.`
        }));
        const cancellationTokenSource = new codegen_csharp_1.Property('CancellationTokenSource', codegen_csharp_1.System.Threading.CancellationTokenSource, { description: 'Accessor for cancellationTokenSource.' });
        cancellationTokenSource.get = (0, codegen_csharp_1.toExpression)(`${cts.value} `);
        cancellationTokenSource.set = new codegen_csharp_1.Statements(cts.assign('value'));
        this.add(cancellationTokenSource);
        this.add(new codegen_csharp_1.LambdaProperty(`${exports_1.ClientRuntime.IEventListener}.Token`, codegen_csharp_1.System.Threading.CancellationToken, new codegen_csharp_1.LiteralExpression(`${cts.value}.Token`), { getAccess: codegen_csharp_1.Access.Default, setAccess: codegen_csharp_1.Access.Default, description: `<see cref="${exports_1.ClientRuntime}.IEventListener" /> cancellation token.` }));
        this.cancellationToken = (0, codegen_csharp_1.toExpression)(`((${exports_1.ClientRuntime.IEventListener})this).Token`);
        this.add(new codegen_csharp_1.LambdaProperty(`${exports_1.ClientRuntime.IEventListener}.Cancel`, codegen_csharp_1.System.Action(), new codegen_csharp_1.LiteralExpression(`${cts.value}.Cancel`), { getAccess: codegen_csharp_1.Access.Default, setAccess: codegen_csharp_1.Access.Default, description: `<see cref="${exports_1.ClientRuntime}.IEventListener" /> cancellation delegate. Stops the cmdlet when called.` }));
        const id = new codegen_csharp_1.Parameter('id', codegen_csharp_1.dotnet.String, { description: 'The message id' });
        const token = new codegen_csharp_1.Parameter('token', codegen_csharp_1.System.Threading.CancellationToken, { description: 'The message cancellation token. When this call is cancelled, this should be <c>true</c>' });
        const messageData = new codegen_csharp_1.Parameter('messageData', codegen_csharp_1.System.Func(exports_1.ClientRuntime.EventData), { description: 'Detailed message data for the message event.' });
        const signalMethod = this.add(new codegen_csharp_1.Method(`${exports_1.ClientRuntime.IEventListener}.Signal`, codegen_csharp_1.System.Threading.Tasks.Task(), {
            async: codegen_csharp_1.Modifier.Async,
            parameters: [id, token, messageData],
            access: codegen_csharp_1.Access.Default,
            description: 'Handles/Dispatches events during the call to the REST service.',
            returnsDescription: `A <see cref="${codegen_csharp_1.System.Threading.Tasks.Task()}" /> that will be complete when handling of the message is completed.`
        }));
        signalMethod.push((0, codegen_csharp_1.Using)('NoSynchronizationContext', ''));
        signalMethod.add(function* () {
            yield (0, codegen_csharp_1.If)(`${token.value}.IsCancellationRequested`, (0, codegen_csharp_1.Return)());
            // if the
            const sw = (0, codegen_csharp_1.Switch)(id, [
                (0, codegen_csharp_1.TerminalCase)(powershell_declarations_1.Events.Verbose.value, function* () {
                    yield `WriteVerbose($"{(messageData().Message ?? ${codegen_csharp_1.System.String.Empty})}");`;
                    yield (0, codegen_csharp_1.Return)();
                }),
                (0, codegen_csharp_1.TerminalCase)(powershell_declarations_1.Events.Warning.value, function* () {
                    yield `WriteWarning($"{(messageData().Message ?? ${codegen_csharp_1.System.String.Empty})}");`;
                    yield (0, codegen_csharp_1.Return)();
                }),
                (0, codegen_csharp_1.TerminalCase)(powershell_declarations_1.Events.Information.value, function* () {
                    if ($this.operation.asjob) {
                        yield '// When an operation supports asjob, Information messages must go thru verbose.';
                        yield `WriteVerbose($"INFORMATION: {(messageData().Message ?? ${codegen_csharp_1.System.String.Empty})}");`;
                    }
                    else {
                        const data = new codegen_csharp_1.LocalVariable('data', codegen_csharp_1.dotnet.Var, { initializer: new codegen_csharp_1.LiteralExpression(`${messageData.use}()`) });
                        yield data.declarationStatement;
                        yield 'WriteInformation(data.Message, new string[]{});';
                    }
                    yield (0, codegen_csharp_1.Return)();
                }),
                (0, codegen_csharp_1.TerminalCase)(powershell_declarations_1.Events.Debug.value, function* () {
                    yield `WriteDebug($"{(messageData().Message ?? ${codegen_csharp_1.System.String.Empty})}");`;
                    yield (0, codegen_csharp_1.Return)();
                }),
                (0, codegen_csharp_1.TerminalCase)(powershell_declarations_1.Events.Error.value, function* () {
                    yield 'WriteError(new global::System.Management.Automation.ErrorRecord( new global::System.Exception(messageData().Message), string.Empty, global::System.Management.Automation.ErrorCategory.NotSpecified, null ) );';
                    yield (0, codegen_csharp_1.Return)();
                }),
                (0, codegen_csharp_1.TerminalCase)(powershell_declarations_1.Events.Progress.value, function* () {
                    yield 'var data = messageData();';
                    yield 'int progress = (int)data.Value;';
                    yield 'string activityMessage, statusDescription;';
                    yield 'global::System.Management.Automation.ProgressRecordType recordType;';
                    yield 'if (progress < 100)';
                    yield '{';
                    yield '    activityMessage = "In progress";';
                    yield '    statusDescription = "Checking operation status";';
                    yield '    recordType = System.Management.Automation.ProgressRecordType.Processing;';
                    yield '}';
                    yield 'else';
                    yield '{';
                    yield '    activityMessage = "Completed";';
                    yield '    statusDescription = "Completed";';
                    yield '    recordType = System.Management.Automation.ProgressRecordType.Completed;';
                    yield '}';
                    // hardcode id = 1 because there is no need for nested progress bar
                    yield 'WriteProgress(new global::System.Management.Automation.ProgressRecord(1, activityMessage, statusDescription)';
                    yield '{';
                    yield '    PercentComplete = progress,';
                    yield 'RecordType = recordType';
                    yield '});';
                    yield (0, codegen_csharp_1.Return)();
                }),
            ]);
            if ($this.operation.asjob) {
                // if we support -AsJob, we support -NoWait
                sw.add((0, codegen_csharp_1.Case)(powershell_declarations_1.Events.DelayBeforePolling.value, function* () {
                    yield 'var data = messageData();';
                    yield (0, codegen_csharp_1.If)('true == MyInvocation?.BoundParameters?.ContainsKey("NoWait")', function* () {
                        yield 'if (data.ResponseMessage is System.Net.Http.HttpResponseMessage response)';
                        yield '{';
                        yield '    var asyncOperation = response.GetFirstHeader(@"Azure-AsyncOperation");';
                        yield '    var location = response.GetFirstHeader(@"Location");';
                        yield '    var uri = global::System.String.IsNullOrEmpty(asyncOperation) ? global::System.String.IsNullOrEmpty(location) ? response.RequestMessage.RequestUri.AbsoluteUri : location : asyncOperation;';
                        yield `    WriteObject(new ${exports_1.ClientRuntime}.PowerShell.AsyncOperationResponse { Target = uri });`;
                        yield '    // do nothing more. ';
                        yield '    data.Cancel();';
                        yield '    return;';
                        yield '}';
                    });
                    yield (0, codegen_csharp_1.Else)(function* () {
                        yield 'if (data.ResponseMessage is System.Net.Http.HttpResponseMessage response)';
                        yield '{';
                        yield '    int delay = (int)(response.Headers.RetryAfter?.Delta?.TotalSeconds ?? 30);';
                        yield '    WriteDebug($"Delaying {delay} seconds before polling.");';
                        yield '    for (var now = 0; now < delay; ++now)';
                        yield '    {';
                        // hardcode id = 1 because there is no need for nested progress bar
                        yield '        WriteProgress(new global::System.Management.Automation.ProgressRecord(1, "In progress", "Checking operation status")';
                        yield '        {';
                        yield '            PercentComplete = now * 100 / delay';
                        yield '        });';
                        yield `        await global::System.Threading.Tasks.Task.Delay(1000, ${token.use});`;
                        yield '    }';
                        yield '}';
                    });
                }));
            }
            // the whole switch statement
            yield sw;
            if ($this.state.project.azure) {
                // in azure mode, we signal the AzAccount module with every event that makes it here.
                yield `await ${$this.state.project.serviceNamespace.moduleClass.declaration}.Instance.Signal(${id.value}, ${token.value}, ${messageData.value}, (i, t, m) => ((${exports_1.ClientRuntime.IEventListener})this).Signal(i, t, () => ${exports_1.ClientRuntime.EventDataConverter}.ConvertFrom(m()) as ${exports_1.ClientRuntime.EventData}), ${$this.invocationInfo.value}, this.ParameterSetName, ${$this.correlationId.value}, ${$this.processRecordId.value}, null );`;
                yield (0, codegen_csharp_1.If)(`${token.value}.IsCancellationRequested`, (0, codegen_csharp_1.Return)());
            }
            else {
                // In Non-Azure Modes, emit the Signal method without coorelation and processrecordid
                yield `await ${$this.state.project.serviceNamespace.moduleClass.declaration}.Instance.Signal(${id.value}, ${token.value}, ${messageData.value}, (i, t, m) => ((${exports_1.ClientRuntime.IEventListener})this).Signal(i, t, () => ${exports_1.ClientRuntime.EventDataConverter}.ConvertFrom(m()) as ${exports_1.ClientRuntime.EventData}), ${$this.invocationInfo.value}, this.ParameterSetName, null ); `;
                yield (0, codegen_csharp_1.If)(`${token.value}.IsCancellationRequested`, (0, codegen_csharp_1.Return)());
            }
            yield `WriteDebug($"{id}: {(messageData().Message ?? ${codegen_csharp_1.System.String.Empty})}");`;
            // any handling of the signal on our side...
        });
    }
    NewAddPowershellParameters(operation) {
        const vps = operation.details.csharp.virtualParameters || {
            body: [],
            operation: [],
        };
        for (const parameter of (0, linq_1.values)(operation.parameters)) {
            // these are the parameters that this command expects
            parameter.schema;
            const td = this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(parameter.schema, true, this.state);
            if (parameter.details.csharp.constantValue) {
                // this parameter has a constant value -- SKIP IT
                continue;
            }
            if (parameter.details.csharp.fromHost) {
                // the parameter is expected to be gotten from the host.(ie, Az.Accounts)
                const hostParameter = this.add(new codegen_csharp_1.BackedProperty(parameter.details.csharp.name, td, {
                    metadata: {
                        parameterDefinition: parameter.details.csharp.httpParameter
                    },
                    description: parameter.details.csharp.description,
                }));
                this.thingsToSerialize.push(hostParameter);
                // in the BeginProcessing, we should tell it to go get the value for this property from the common module
                this.$('BeginProcessing').add(hostParameter.assignPrivate(new codegen_csharp_1.LiteralExpression(`${this.state.project.serviceNamespace.moduleClass.declaration}.Instance.GetParameter(this.MyInvocation, ${this.correlationId.value}, "${parameter.name}") as string`)));
                continue;
            }
            const $this = this;
            if (parameter.details.csharp.apiversion) {
                // Api-version parameters for azure are a custom implementation
                this.add(new codegen_csharp_1.Property(parameter.details.csharp.name, td, {
                    getAccess: codegen_csharp_1.Access.Internal,
                    setAccess: codegen_csharp_1.Access.Private,
                    metadata: {
                        parameterDefinition: parameter.details.csharp.httpParameter
                    },
                    description: parameter.details.csharp.description,
                    *get() {
                        const metadata = operation.extensions['x-ms-metadata'];
                        const profiles = metadata.profiles || new linq_1.Dictionary();
                        yield (0, codegen_csharp_1.Switch)(`${$this.state.project.serviceNamespace.moduleClass.declaration}.Instance.Profile`, function* () {
                            for (const { key: profileName, value: apiVersion } of (0, linq_1.items)(profiles)) {
                                yield (0, codegen_csharp_1.TerminalCase)(`"${profileName}"`, (0, codegen_csharp_1.Return)(`"${apiVersion}"`));
                            }
                            yield (0, codegen_csharp_1.TerminalDefaultCase)((0, codegen_csharp_1.Return)(`"${metadata.apiVersions[0]}"`));
                        });
                    }
                }));
                continue;
            }
            if (this.dropBodyParameter && parameter.details.csharp.isBodyParameter) {
                // we're supposed to use parameters for the body parameter instead of a big object
                const expandedBodyParameter = this.add(new codegen_csharp_1.Field('_' + (0, codegen_1.camelCase)(parameter.details.csharp.name), td, {
                    description: parameter.details.csharp.description,
                    initialValue: (parameter.schema.type === codemodel_1.SchemaType.Array) ? codegen_csharp_1.dotnet.Null : `new ${parameter.schema.language.csharp?.fullname}()`,
                    access: codegen_csharp_1.Access.Private
                }));
                this.thingsToSerialize.push(expandedBodyParameter);
                for (const vParam of vps.body) {
                    const vSchema = vParam.schema;
                    vParam.origin;
                    const propertyType = this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(vSchema, true, this.state);
                    // we need to know if the actual underlying property is actually nullable.
                    const nullable = this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(vSchema, !!vParam.origin.required, this.state).isNullable;
                    let cmdletParameter;
                    if (propertyType.schema.type !== codemodel_1.SchemaType.Array) {
                        cmdletParameter = new codegen_csharp_1.Property(vParam.name, propertyType, {
                            get: (0, codegen_csharp_1.toExpression)(`${expandedBodyParameter.value}.${(0, model_class_1.getVirtualPropertyName)(vParam.origin) || vParam.origin.name}${!nullable ? '' : ` ?? ${propertyType.defaultOfType}`}`), // /* ${inspect(vParam.origin)} */
                            // get: toExpression(`null == ${expandedBodyParameter.value}.${vParam.origin.name} ? ${propertyType.defaultOfType} : (${propertyType.declaration}) ${expandedBodyParameter.value}.${vParam.origin.name}`),
                            set: (0, codegen_csharp_1.toExpression)(`${expandedBodyParameter.value}.${(0, model_class_1.getVirtualPropertyName)(vParam.origin) || vParam.origin.name} = value`),
                            new: PropertiesRequiringNew.has(vParam.name) ? codegen_csharp_1.Modifier.New : codegen_csharp_1.Modifier.None
                        });
                    }
                    else {
                        const fixedArrayPropertyType = this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(vSchema, true, this.state, true);
                        cmdletParameter = new codegen_csharp_1.Property(vParam.name, fixedArrayPropertyType, {
                            get: (0, codegen_csharp_1.toExpression)(`${expandedBodyParameter.value}.${(0, model_class_1.getVirtualPropertyName)(vParam.origin) || vParam.origin.name}?.ToArray()${` ?? ${fixedArrayPropertyType.defaultOfType}`}`),
                            set: (0, codegen_csharp_1.toExpression)(`${expandedBodyParameter.value}.${(0, model_class_1.getVirtualPropertyName)(vParam.origin) || vParam.origin.name} = (value != null ? new ${propertyType.declaration}(value) : null)`),
                            new: PropertiesRequiringNew.has(vParam.name) ? codegen_csharp_1.Modifier.New : codegen_csharp_1.Modifier.None
                        });
                    }
                    if (vParam.schema.language.csharp?.byReference) {
                        // this parameter's schema is marked as 'by-reference' which means we should
                        // tag it with an ExportAs attribute for the I*Reference type.
                        cmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ExportAsAttribute, { parameters: [`typeof(${vParam.schema.language.csharp.referenceInterface})`] }));
                    }
                    if (vParam.schema.type === codemodel_1.SchemaType.Array) {
                        //skip-for-time-being
                        // if ((<ArraySchema>vParam.schema). && vParam.schema.items.details.csharp.byReference) {
                        //   cmdletParameter.add(new Attribute(ExportAsAttribute, { parameters: [`typeof(${vParam.schema.items.details.csharp.referenceInterface}[])`] }));
                        // }
                        cmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.AllowEmptyCollectionAttribute));
                    }
                    const dictSchema = vSchema.type === codemodel_1.SchemaType.Dictionary ? vSchema :
                        vSchema.type === codemodel_1.SchemaType.Object ? vSchema.parents?.immediate?.find((s) => s.type === codemodel_1.SchemaType.Dictionary) :
                            undefined;
                    if (dictSchema) {
                        // we have to figure out if this is a standalone dictionary or a hybrid object/dictionary.
                        // if it's a hybrid, we have to create another parameter like -<XXX>AdditionalProperties and have that dump the contents into the dictionary
                        // if it's a standalone dictionary, we can just use hashtable instead
                        if ((0, linq_1.length)(vSchema.properties) === 0) {
                            // it's a pure dictionary
                            // add an attribute for changing the exported type.
                            cmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ExportAsAttribute, { parameters: [`typeof(${codegen_csharp_1.System.Collections.Hashtable})`] }));
                        }
                        else {
                            // it's a hybrid. We need to add an additional property that puts its items into the target container
                        }
                    }
                    const desc = (vParam.description || '.').replace(/[\r?\n]/gm, '');
                    cmdletParameter.description = desc;
                    // check if this parameter is a byte array, which would indicate that it should really be a file input
                    if (cmdletParameter.type.declaration === codegen_csharp_1.dotnet.Binary.declaration) {
                        // set the generated parameter to internal
                        cmdletParameter.setAccess = codegen_csharp_1.Access.Internal;
                        cmdletParameter.getAccess = codegen_csharp_1.Access.Internal;
                        // create a InputFileXXX for the parameter
                        const ifname = vParam.name.toLowerCase() === 'value' ? 'InputFile' : (0, codegen_1.pascalCase)([vParam.name, 'Input', 'File']);
                        const inputFileParameter = new codegen_csharp_1.Property(ifname, codegen_csharp_1.dotnet.String, {
                            // get: toExpression(`${expandedBodyParameter.value}.${vParam.origin.name}${vParam.required ? '' : ` ?? ${propertyType.defaultOfType}`}`),
                            set: function* () {
                                const provider = (0, codegen_csharp_2.Local)('provider');
                                provider.initializer = undefined;
                                const paths = (0, codegen_csharp_2.Local)('paths', `this.SessionState.Path.GetResolvedProviderPathFromPSPath(value, out ${provider.declarationExpression})`);
                                yield paths.declarationStatement;
                                yield (0, codegen_csharp_1.If)(`${provider.value}.Name != "FileSystem" || ${paths.value}.Count == 0`, 'ThrowTerminatingError( new System.Management.Automation.ErrorRecord(new global::System.Exception("Invalid input path."),string.Empty, global::System.Management.Automation.ErrorCategory.InvalidArgument, value) );');
                                yield (0, codegen_csharp_1.If)(`${paths.value}.Count > 1`, 'ThrowTerminatingError( new System.Management.Automation.ErrorRecord(new global::System.Exception("Multiple input paths not allowed."),string.Empty, global::System.Management.Automation.ErrorCategory.InvalidArgument, value) );');
                                yield cmdletParameter.assign(`global::System.IO.File.ReadAllBytes(${paths.value}[0])`);
                            },
                            description: `Input File for ${cmdletParameter.name} (${(0, codegen_1.escapeString)(desc)})`
                        });
                        inputFileParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: [new codegen_csharp_1.LiteralExpression(`Mandatory = ${vParam.required ? 'true' : 'false'}`), new codegen_csharp_1.LiteralExpression(`HelpMessage = "Input File for ${cmdletParameter.name} (${(0, codegen_1.escapeString)(desc || '.')})"`)] }));
                        inputFileParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Body`] }));
                        if ((0, linq_1.length)(vParam.alias) > 0) {
                            inputFileParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.Alias, { parameters: vParam.alias.map(x => '"' + x + '"') }));
                        }
                        $this.add(inputFileParameter);
                    }
                    else {
                        cmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: [new codegen_csharp_1.LiteralExpression(`Mandatory = ${vParam.required ? 'true' : 'false'}`), new codegen_csharp_1.LiteralExpression(`HelpMessage = "${(0, codegen_1.escapeString)(desc || '.')}"`)] }));
                        cmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Body`] }));
                        NewAddInfoAttribute(cmdletParameter, propertyType, !!vParam.required, false, desc, vParam.origin.property.serializedName);
                        NewAddCompleterInfo(cmdletParameter, vParam);
                        addParameterBreakingChange(cmdletParameter, vParam);
                        addParameterPreviewMessage(cmdletParameter, vParam);
                        addDefaultInfo(cmdletParameter, vParam);
                        this.addDoNotExport(cmdletParameter, vParam);
                    }
                    const addArgumentCompleter = isEnumImplementation(vParam.schema) || propertyType instanceof exports_1.ArrayOf && isEnumImplementation(propertyType.elementType.schema);
                    if (addArgumentCompleter) {
                        addPSArgumentCompleterAttribute(cmdletParameter, vParam.schema);
                    }
                    // add aliases if there is any
                    if ((0, linq_1.length)(vParam.alias) > 0) {
                        cmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.Alias, { parameters: vParam.alias.map(x => '"' + x + '"') }));
                    }
                    this.add(cmdletParameter);
                }
                const paramDictSchema = parameter.schema.type === codemodel_1.SchemaType.Dictionary ? parameter.schema :
                    parameter.schema.type === codemodel_1.SchemaType.Object ? parameter.schema.parents?.immediate?.find((s) => s.type === codemodel_1.SchemaType.Dictionary) :
                        undefined;
                if (paramDictSchema) {
                    // if there is an additional properties on this type
                    // add a hashtable parameter for additionalProperties
                    let apPropName = '';
                    const options = ['AdditionalProperties', 'MoreProperties', 'ExtendedProperties', 'Properties'];
                    for (const n of options) {
                        if (this.properties.find(each => each.name === n)) {
                            continue;
                        }
                        apPropName = n;
                        break;
                    }
                    this.apProp = this.add(new codegen_csharp_1.Property(apPropName, codegen_csharp_1.System.Collections.Hashtable));
                    this.apProp.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, {
                        parameters: ['Mandatory = false', 'HelpMessage = "Additional Parameters"']
                    }));
                    this.bodyParameterInfo = {
                        type: {
                            declaration: parameter.schema.language.csharp?.fullname
                        },
                        valueType: paramDictSchema.elementType.type === codemodel_1.SchemaType.Any ? codegen_csharp_1.System.Object :
                            this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(paramDictSchema.elementType, true, this.state)
                    };
                }
                this.bodyParameter = expandedBodyParameter;
                continue;
            }
        }
        if (this.isViaIdentity) {
            const viaIdentityRegex = /ViaIdentity\d?/g;
            this.inputObjectParameterName = `${this.name.split(viaIdentityRegex)[1].split('Expanded')[0]}InputObject`;
            // add in the pipeline parameter for the identity
            const idschema = (0, linq_1.values)(this.state.project.model.schemas.objects).first(each => each.language.default.uid === 'universal-parameter-type');
            const idtd = this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(idschema, true, this.state);
            const idParam = this.add(new codegen_csharp_1.BackedProperty(this.inputObjectParameterName, idtd, {
                description: 'Identity Parameter'
            }));
            const parameters = [new codegen_csharp_1.LiteralExpression('Mandatory = true'), new codegen_csharp_1.LiteralExpression('HelpMessage = "Identity Parameter"'), new codegen_csharp_1.LiteralExpression('ValueFromPipeline = true')];
            idParam.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters }));
            idParam.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Path`] }));
        }
        for (const vParam of (0, linq_1.values)(vps.operation)) {
            if (vParam.name === 'Host') {
                // skip 'Host'
                continue;
            }
            let regularCmdletParameter;
            let origin = null;
            let propertyType = null;
            if (vParam.type) {
                // Handle parameters added through directives
                regularCmdletParameter = this.add(new codegen_csharp_1.BackedProperty(vParam.name, new codegen_csharp_1.ClassType('', vParam.type), {
                    description: vParam.description
                }));
            }
            else {
                const vSchema = vParam.schema;
                propertyType = this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(vSchema, true, this.state);
                origin = vParam.origin;
                regularCmdletParameter = (this.state.project.azure && vParam.name === 'SubscriptionId' && operation.details.csharp.verb.toLowerCase() === 'get') ?
                    // special case for subscription id
                    this.add(new codegen_csharp_1.BackedProperty(vParam.name, codegen_csharp_1.dotnet.StringArray, {
                        metadata: {
                            parameterDefinition: origin.details.csharp.httpParameter
                        },
                        description: vParam.description
                    })) :
                    // everything else
                    this.add(new codegen_csharp_1.BackedProperty(vParam.name, propertyType, {
                        metadata: {
                            parameterDefinition: origin.details.csharp.httpParameter
                        },
                        description: vParam.description
                    }));
                const dictSchema = vSchema.type === codemodel_1.SchemaType.Dictionary ? vSchema :
                    vSchema.type === codemodel_1.SchemaType.Object ? vSchema.parents?.immediate?.find((s) => s.type === codemodel_1.SchemaType.Dictionary) :
                        undefined;
                if (dictSchema) {
                    // we have to figure out if this is a standalone dictionary or a hybrid object/dictionary.
                    // if it's a hybrid, we have to create another parameter like -<XXX>AdditionalProperties and have that dump the contents into the dictionary
                    // if it's a standalone dictionary, we can just use hashtable instead
                    if ((0, linq_1.length)(vSchema.properties) === 0) {
                        // it's a pure dictionary
                        // change the property type to hashtable.
                        // add an attribute to change the exported type.
                        regularCmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ExportAsAttribute, { parameters: [`typeof(${codegen_csharp_1.System.Collections.Hashtable})`] }));
                    }
                    else {
                        // it's a hybrid. We need to add an additional property that puts its items into the target container
                    }
                }
            }
            this.thingsToSerialize.push(regularCmdletParameter);
            const parameters = [new codegen_csharp_1.LiteralExpression(`Mandatory = ${vParam.required ? 'true' : 'false'}`), new codegen_csharp_1.LiteralExpression(`HelpMessage = "${(0, codegen_1.escapeString)(vParam.description) || '.'}"`)];
            if (!!origin && origin.details.csharp.isBodyParameter) {
                parameters.push(new codegen_csharp_1.LiteralExpression('ValueFromPipeline = true'));
                this.bodyParameter = regularCmdletParameter;
            }
            regularCmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters }));
            if (!vParam.type && vParam.schema.type === codemodel_1.SchemaType.Array) {
                regularCmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.AllowEmptyCollectionAttribute));
            }
            if (!!origin && !!propertyType) {
                NewAddInfoAttribute(regularCmdletParameter, propertyType, vParam.required ?? false, false, vParam.description, origin.name);
            }
            NewAddCompleterInfo(regularCmdletParameter, vParam);
            addParameterBreakingChange(regularCmdletParameter, vParam);
            addParameterPreviewMessage(regularCmdletParameter, vParam);
            addDefaultInfo(regularCmdletParameter, vParam);
            this.addDoNotExport(regularCmdletParameter, vParam);
            // add aliases if there is any
            if ((0, linq_1.length)(vParam.alias) > 0) {
                regularCmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.Alias, { parameters: vParam.alias.map(x => '"' + x + '"') }));
            }
            const httpParam = origin ? origin.details.csharp.httpParameter : null;
            //const uid = httpParam ? httpParam.details.csharp.uid : 'no-parameter';
            if (httpParam) {
                // xichen: Is it safe to compare by csharp serializedName? Because we no longer have uid
                const cat = this.apiCall.parameters?.find((param) => !param.language.csharp?.constantValue && param.language.csharp?.serializedName === httpParam.language.csharp?.serializedName);
                if (cat) {
                    regularCmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.${(0, codegen_1.pascalCase)((cat.protocol.http?.in))}`] }));
                }
            }
            else {
                const isBodyParameter = origin ? origin.details.csharp.isBodyParameter : false;
                if (isBodyParameter) {
                    regularCmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, {
                        parameters: [`${powershell_declarations_1.ParameterCategory}.Body`],
                    }));
                }
            }
            const addArgumentCompleter = isEnumImplementation(vParam.schema) || propertyType instanceof exports_1.ArrayOf && isEnumImplementation(propertyType.elementType.schema);
            if (addArgumentCompleter) {
                addPSArgumentCompleterAttribute(regularCmdletParameter, vParam.schema);
            }
        }
        const ifmatch = this.properties.find((v) => v.name.toLowerCase() === 'ifmatch');
        if (ifmatch) {
            //no sure why there is an empty block
        }
    }
    /**
     * Add `DoNotExportAttribute` to parameters that should be hidden.
     * @param cmdletParameter parameter in the variant.
     * @param vParam parameter in the code model.
     * @see DoNotExportAttribute
     */
    addDoNotExport(cmdletParameter, vParam) {
        if (vParam.hidden) {
            this.state.message({
                Channel: extension_base_1.Channel.Debug,
                Text: `[DIRECTIVE] Applied 'hide' directive to parameter ${cmdletParameter.name}. Added attribute ${powershell_declarations_1.DoNotExportAttribute.declaration} to parameter.`,
            });
            cmdletParameter.add(new codegen_csharp_1.Attribute(powershell_declarations_1.DoNotExportAttribute));
        }
    }
    NewAddClassAttributes(operation, variantName) {
        const cmdletAttribParams = [
            category[operation.details.csharp.verb] ? (0, powershell_declarations_1.verbEnum)(category[operation.details.csharp.verb], operation.details.csharp.verb) : `"${operation.details.csharp.verb}"`,
            new codegen_csharp_1.StringExpression(variantName)
        ];
        if ((0, command_operation_1.isWritableCmdlet)(operation) && !operation.details.csharp.supportShouldProcess) {
            cmdletAttribParams.push('SupportsShouldProcess = true');
        }
        if (this.clientsidePagination) {
            cmdletAttribParams.push('SupportsPaging = true');
        }
        if (operation.details.csharp.hidden) {
            this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.InternalExportAttribute));
            const noun = `${operation.details.csharp.subjectPrefix}${operation.details.csharp.subject}`;
            const cmdletName = `${operation.details.csharp.verb}-${noun}${operation.details.csharp.name ? `_${operation.details.csharp.name}` : ''}`;
            this.state.message({ Channel: extension_base_1.Channel.Debug, Text: `[DIRECTIVE] Applied 'hide' directive to ${cmdletName}. Added attribute ${powershell_declarations_1.InternalExportAttribute.declaration} to cmdlet.` });
        }
        this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CmdletAttribute, { parameters: cmdletAttribParams }));
        // add alias attribute if there is any aliases for this cmdlet
        if ((0, linq_1.length)(operation.details.csharp.alias) > 0) {
            this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.Alias, { parameters: operation.details.csharp.alias.map((x) => '"' + x + '"') }));
        }
        let shouldAddPassThru = false;
        // set to hold the output types
        const outputTypes = new Set();
        for (const httpOperation of (0, linq_1.values)(operation.callGraph)) {
            const pageableInfo = httpOperation.language.csharp?.pageable;
            const v = httpOperation.responses && httpOperation.responses.length > 0 && httpOperation.responses[0] instanceof codemodel_1.SchemaResponse;
            // Add this for binary response in m4
            for (const binary of (0, linq_1.values)(httpOperation.responses).selectNonNullable(each => each.binary)) {
                if (binary) {
                    // if this is a stream, skip the output type.
                    this.hasStreamOutput = true;
                    shouldAddPassThru = true;
                    outputTypes.add(`typeof(${codegen_csharp_1.dotnet.Bool})`);
                }
            }
            for (const schema of (0, linq_1.values)(httpOperation.responses).selectNonNullable(each => each.schema)) {
                const props = (0, schema_1.getAllProperties)(schema);
                // does the target type just wrap a single output?
                const resultSchema = schema;
                // make sure return type for boolean stays boolean!
                if (resultSchema.type === codemodel_1.SchemaType.Boolean ||
                    (resultSchema.type === codemodel_1.SchemaType.Choice && resultSchema.choiceType.type === codemodel_1.SchemaType.Boolean && resultSchema.choices.length === 1) ||
                    (resultSchema.type === codemodel_1.SchemaType.SealedChoice && resultSchema.choiceType.type === codemodel_1.SchemaType.Boolean && resultSchema.choices.length === 1)) {
                    outputTypes.add(`typeof(${codegen_csharp_1.dotnet.Bool})`);
                }
                else {
                    const typeDeclaration = this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(resultSchema, true, this.state);
                    if (typeDeclaration.declaration === codegen_csharp_1.System.IO.Stream.declaration || typeDeclaration.declaration === codegen_csharp_1.dotnet.Binary.declaration) {
                        // if this is a stream, skip the output type.
                        this.hasStreamOutput = true;
                        shouldAddPassThru = true;
                        outputTypes.add(`typeof(${codegen_csharp_1.dotnet.Bool})`);
                    }
                    else {
                        let type = '';
                        if (typeDeclaration instanceof exports_1.ArrayOf) {
                            type = typeDeclaration.elementTypeDeclaration;
                        }
                        else if (pageableInfo && pageableInfo.responseType === 'pageable') {
                            if (typeDeclaration === undefined || typeDeclaration.schema.properties?.find(p => p.serializedName === pageableInfo.itemName) === undefined) {
                                //skip-for-time-being, since operationId does not support in m4 any more
                                //throw new Error(`\n\nOn operation:\n  '${httpOperation.operationId}' at '${httpOperation.path}'\n  -- you have used 'x-ms-pageable' and there is no property name '${pageableInfo.itemName}' that is an array.\n\n`);
                                throw new Error('An error needs to be more specific');
                            }
                            const nestedSchema = typeDeclaration.schema.properties?.find(p => p.serializedName === pageableInfo.itemName)?.schema;
                            const nestedTypeDeclaration = this.state.project.schemaDefinitionResolver.resolveTypeDeclaration(nestedSchema, true, this.state);
                            type = nestedTypeDeclaration.elementTypeDeclaration;
                        }
                        else {
                            type = typeDeclaration.declaration;
                        }
                        // check if this is a stream output
                        if (type) {
                            outputTypes.add(`typeof(${type})`);
                        }
                    }
                }
            }
        }
        // if any response does not return,
        // the cmdlet should have a PassThru parameter
        shouldAddPassThru = shouldAddPassThru || (0, linq_1.values)(operation.callGraph)
            .selectMany(httpOperation => (0, linq_1.values)((httpOperation.responses || []).concat(httpOperation.exceptions || [])))
            //.selectMany(responsesItem => responsesItem.value)
            .any(value => value.schema === undefined);
        if (outputTypes.size === 0) {
            outputTypes.add(`typeof(${codegen_csharp_1.dotnet.Bool})`);
        }
        //add breaking change attributes for cmdlet, variant, output type
        if (operation.details.csharp.breakingChange) {
            const breakingChange = operation.details.csharp.breakingChange;
            if (breakingChange.cmdlet) {
                const parameters = [];
                if (!breakingChange.cmdlet.deprecateByVersion || !breakingChange.cmdlet.deprecateByAzVersion) {
                    throw new Error('Cmdlet breaking change requires both \'deprecateByVersion\' and \'deprecateByAzVersion\', please refer to https://github.com/Azure/azure-powershell/blob/main/documentation/development-docs/breakingchange-for-autogen-module.md for more details.');
                }
                parameters.push(`"${breakingChange.cmdlet.deprecateByAzVersion}"`);
                parameters.push(`"${breakingChange.cmdlet.deprecateByVersion}"`);
                if (breakingChange.cmdlet.changeInEfectByDate)
                    parameters.push(`"${breakingChange.cmdlet.changeInEfectByDate}"`);
                if (breakingChange.cmdlet.replacement)
                    parameters.push(`ReplacementCmdletName = "${breakingChange.cmdlet.replacement}"`);
                if (breakingChange.cmdlet.changeDescription)
                    parameters.push(`ChangeDescription = "${breakingChange.cmdlet.changeDescription}"`);
                this.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.CmdletBreakingChangeAttribute, { parameters: parameters }));
            }
            if (breakingChange.variant) {
                const parameters = [];
                parameters.push(`new string[] {"${breakingChange.variant.name}"}`);
                if (!breakingChange.variant.deprecateByVersion || !breakingChange.variant.deprecateByAzVersion) {
                    throw new Error('Cmdlet breaking change requires both \'deprecateByVersion\' and \'deprecateByAzVersion\', please refer to https://github.com/Azure/azure-powershell/blob/main/documentation/development-docs/breakingchange-for-autogen-module.md for more details.');
                }
                parameters.push(`"${breakingChange.variant.deprecateByAzVersion}"`);
                parameters.push(`"${breakingChange.variant.deprecateByVersion}"`);
                if (breakingChange.variant.changeInEfectByDate)
                    parameters.push(`"${breakingChange.variant.changeInEfectByDate}"`);
                if (breakingChange.variant.changeDescription)
                    parameters.push(`ChangeDescription = "${breakingChange.variant.changeDescription}"`);
                this.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.ParameterSetBreakingChangeAttribute, { parameters: parameters }));
            }
            if (breakingChange.output) {
                const parameters = [];
                // if deprecated output types are set in directive, use it
                if (breakingChange.output.deprecatedCmdLetOutputType) {
                    parameters.push(`"${breakingChange.output.deprecatedCmdLetOutputType}"`);
                }
                else {
                    parameters.push(`"${outputTypes.values().next().value.replace(/typeof\((.*)\)/, '$1')}"`);
                }
                if (!breakingChange.output.deprecateByVersion || !breakingChange.output.deprecateByAzVersion) {
                    throw new Error('Cmdlet breaking change requires both \'deprecateByVersion\' and \'deprecateByAzVersion\', please refer to https://github.com/Azure/azure-powershell/blob/main/documentation/development-docs/breakingchange-for-autogen-module.md for more details.');
                }
                parameters.push(`"${breakingChange.output.deprecateByAzVersion}"`);
                parameters.push(`"${breakingChange.output.deprecateByVersion}"`);
                if (breakingChange.output.changeInEfectByDate)
                    parameters.push(`"${breakingChange.output.changeInEfectByDate}"`);
                if (breakingChange.output.replacement)
                    parameters.push(`ReplacementCmdletOutputType = "${breakingChange.output.replacement}"`);
                if (breakingChange.output.deprecatedOutputProperties) {
                    const properties = Object.assign([], breakingChange.output.deprecatedOutputProperties);
                    properties.forEach((element, index) => properties[index] = '"' + element + '"');
                    parameters.push(`DeprecatedOutputProperties = new string[] {${properties.join(',')}}`);
                }
                if (breakingChange.output.newOutputProperties) {
                    const properties = Object.assign([], breakingChange.output.newOutputProperties);
                    properties.forEach((element, index) => properties[index] = '"' + element + '"');
                    parameters.push(`NewOutputProperties = new string[] {${properties.join(',')} } `);
                }
                if (breakingChange.output.changeDescription)
                    parameters.push(`ChangeDescription = "${breakingChange.output.changeDescription}"`);
                this.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.OutputBreakingChangeAttribute, { parameters: parameters }));
            }
        }
        //add preview message attribute for cmdlet
        if (operation.details.csharp.previewAnnouncement) {
            const parameters = [];
            parameters.push(`"${operation.details.csharp.previewAnnouncement.previewMessage}"`);
            if (operation.details.csharp.previewAnnouncement.estimatedGaDate)
                parameters.push(`"${operation.details.csharp.previewAnnouncement.estimatedGaDate}"`);
            this.add(new codegen_csharp_1.Attribute(exports_1.ClientRuntime.PreviewMessageAttribute, { parameters: parameters }));
        }
        this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.OutputTypeAttribute, { parameters: [...outputTypes] }));
        if (shouldAddPassThru) {
            const passThru = this.add(new codegen_csharp_1.Property('PassThru', powershell_declarations_1.SwitchParameter, { description: 'When specified, forces the cmdlet return a \'bool\' given that there isn\'t a return type by default.' }));
            passThru.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ParameterAttribute, { parameters: ['Mandatory = false', 'HelpMessage = "Returns true when the command succeeds"'] }));
            passThru.add(new codegen_csharp_1.Attribute(powershell_declarations_1.CategoryAttribute, { parameters: [`${powershell_declarations_1.ParameterCategory}.Runtime`] }));
        }
        this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.DescriptionAttribute, { parameters: [new codegen_csharp_1.StringExpression(this.description)] }));
        // If defines externalDocs for operation
        if (operation.details.default.externalDocs) {
            this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ExternalDocsAttribute, {
                parameters: [`${new codegen_csharp_1.StringExpression(this.operation.details.default.externalDocs?.url ?? '')}`,
                    `${new codegen_csharp_1.StringExpression(this.operation.details.default.externalDocs?.description ?? '')}`]
            }));
        }
        this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.GeneratedAttribute));
        if (operation.extensions && operation.extensions['x-ms-metadata'] && operation.extensions['x-ms-metadata'].profiles) {
            const profileNames = Object.keys(operation.extensions && operation.extensions['x-ms-metadata'].profiles);
            // wrap profile names
            profileNames.forEach((element, index) => {
                profileNames[index] = `"${element}"`;
            });
            this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.ProfileAttribute, { parameters: [...profileNames] }));
        }
        if (this.operation.callGraph.length === 1) {
            this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.HttpPathAttribute, { parameters: [`Path = "${this.apiCall.requests?.[0].protocol?.http?.path}"`, `ApiVersion = "${this.apiCall.apiVersions?.[0].version}"`] }));
        }
        if (variantName.includes('ViaJsonString') || variantName.includes('ViaJsonFilePath')) {
            this.add(new codegen_csharp_1.Attribute(powershell_declarations_1.NotSuggestDefaultParameterSetAttribute));
        }
    }
}
exports.CmdletClass = CmdletClass;
//# sourceMappingURL=class.js.map