"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const extension_base_1 = require("@autorest/extension-base");
const plugin_create_inline_properties_1 = require("./plugins/plugin-create-inline-properties");
const plugin_tweak_model_1 = require("./plugins/plugin-tweak-model");
const plugin_tweak_model_azure_v2_1 = require("./plugins/plugin-tweak-model-azure-v2");
const create_commands_v2_1 = require("./plugins/create-commands-v2");
const cs_namer_v2_1 = require("./plugins/cs-namer-v2");
const ps_namer_v2_1 = require("./plugins/ps-namer-v2");
const llcsharp_v2_1 = require("./plugins/llcsharp-v2");
const powershell_v2_1 = require("./plugins/powershell-v2");
const add_azure_completers_v2_1 = require("./plugins/add-azure-completers-v2");
const modifiers_v2_1 = require("./plugins/modifiers-v2");
const plugin_tweak_m4_model_1 = require("./plugins/plugin-tweak-m4-model");
const sdk_generate_1 = require("./plugins/sdk-generate");
const sdk_tweak_model_1 = require("./plugins/sdk-tweak-model");
const sdk_cs_simplifier_1 = require("./plugins/sdk-cs-simplifier");
const sdk_cs_namer_1 = require("./plugins/sdk-cs-namer");
const sdk_modifiers_1 = require("./plugins/sdk-modifiers");
const sdk_create_inline_properties_1 = require("./plugins/sdk-create-inline-properties");
require('source-map-support').install();
async function main() {
    const pluginHost = new extension_base_1.AutoRestExtension();
    // Following are plugins moved from remodeler
    pluginHost.Add('tweakm4codemodel', plugin_tweak_m4_model_1.tweakM4ModelPlugin);
    pluginHost.Add('tweakcodemodel-v2', plugin_tweak_model_1.tweakModelPlugin);
    pluginHost.Add('tweakcodemodelazure-v2', plugin_tweak_model_azure_v2_1.tweakModelAzurePluginV2);
    pluginHost.Add('create-virtual-properties-v2', plugin_create_inline_properties_1.createInlinedPropertiesPlugin);
    pluginHost.Add('create-commands-v2', create_commands_v2_1.createCommandsV2);
    pluginHost.Add('csnamer-v2', cs_namer_v2_1.csnamerV2);
    pluginHost.Add('psnamer-v2', ps_namer_v2_1.namerV2);
    pluginHost.Add('modifiers-v2', modifiers_v2_1.applyModifiersV2);
    pluginHost.Add('add-azure-completers-v2', add_azure_completers_v2_1.addCompleterV2);
    pluginHost.Add('llcsharp-v2', llcsharp_v2_1.llcsharpV2);
    pluginHost.Add('powershell-v2', powershell_v2_1.powershellV2);
    pluginHost.Add('generate', sdk_generate_1.generate);
    pluginHost.Add('tweakSdkModelPlugin', sdk_tweak_model_1.tweakSdkModelPlugin);
    pluginHost.Add('simplifierPlugin', sdk_cs_simplifier_1.simplifierPlugin);
    pluginHost.Add('csnamerSdk', sdk_cs_namer_1.csnamerSdk);
    pluginHost.Add('applyModifiersSdk', sdk_modifiers_1.applyModifiersSdk);
    pluginHost.Add('applyModelNameModifiersSdk', sdk_modifiers_1.applyModelNameModifiersSdk);
    pluginHost.Add('createSdkInlinedPropertiesPlugin', sdk_create_inline_properties_1.createSdkInlinedPropertiesPlugin);
    await pluginHost.Run();
}
exports.main = main;
main();
//# sourceMappingURL=main.js.map